@@ -11,11 +11,14 @@
 
 EAPI E_Config *e_config = NULL;
 
+static int _e_config_revisions = 0;
+
 /* local subsystem functions */
 static void _e_config_save_cb(void *data);
 static void _e_config_free(E_Config *cfg);
-static int  _e_config_cb_timer(void *data);
-static int  _e_config_eet_close_handle(Eet_File *ef, char *file);
+static Eina_Bool _e_config_cb_timer(void *data);
+static int _e_config_eet_close_handle(Eet_File *ef, char *file);
+static void _e_config_acpi_bindings_add(void);
 
 /* local subsystem globals */
 static int _e_config_save_block = 0;
@@ -32,6 +35,7 @@ static E_Config_DD *_e_config_bindings_key_edd = NULL;
 static E_Config_DD *_e_config_bindings_edge_edd = NULL;
 static E_Config_DD *_e_config_bindings_signal_edd = NULL;
 static E_Config_DD *_e_config_bindings_wheel_edd = NULL;
+static E_Config_DD *_e_config_bindings_acpi_edd = NULL;
 static E_Config_DD *_e_config_path_append_edd = NULL;
 static E_Config_DD *_e_config_desktop_bg_edd = NULL;
 static E_Config_DD *_e_config_desktop_name_edd = NULL;
@@ -47,6 +51,44 @@ static E_Config_DD *_e_config_syscon_action_edd = NULL;
 EAPI int E_EVENT_CONFIG_ICON_THEME = 0;
 EAPI int E_EVENT_CONFIG_MODE_CHANGED = 0;
 
+static char *
+_e_config_profile_name_get(Eet_File *ef)
+{
+   /* profile config exists */
+   char *data, *s = NULL;
+   int data_len = 0;
+   
+   data = eet_read(ef, "config", &data_len);
+   if ((data) && (data_len > 0))
+     {
+        int ok = 1;
+        
+        for (s = data; s < (data + data_len); s++)
+          {
+             // if profile is not all ascii (valid printable ascii - no
+             // control codes etc.) or it contains a '/' (invalid as its a
+             // directory delimiter) - then it's invalid
+             if ((*s < ' ') || (*s > '~') || (*s == '/'))
+               {
+                  ok = 0;
+                  break;
+               }
+          }
+        s = NULL;
+        if (ok)
+          {
+             s = malloc(data_len + 1);
+             if (s)
+               {
+                  memcpy(s, data, data_len);
+                  s[data_len] = 0;
+               }
+          }
+        free(data);
+     }
+   return s;
+}
+
 /* externally accessible functions */
 EAPI int
 e_config_init(void)
@@ -66,32 +108,42 @@ e_config_init(void)
 	/* try user profile config */
 	e_user_dir_concat_static(buf, "config/profile.cfg");
 	ef = eet_open(buf, EET_FILE_MODE_READ);
-	if (!ef)
+        if (ef)
+          {
+             _e_config_profile = _e_config_profile_name_get(ef);
+             eet_close(ef);
+             ef = NULL;
+          }
+	if (!_e_config_profile)
 	  {
-	     /* use system if no user profile config */
-	     e_prefix_data_concat_static(buf, "data/config/profile.cfg");
-	     ef = eet_open(buf, EET_FILE_MODE_READ);
+             int i;
+             
+             for (i =1; i <= _e_config_revisions; i++)
+               {
+                  e_user_dir_snprintf(buf, sizeof(buf), "config/profile.%i.cfg", i);
+                  ef = eet_open(buf, EET_FILE_MODE_READ);
+                  if (ef)
+                    {
+                       _e_config_profile = _e_config_profile_name_get(ef);
+                       eet_close(ef);
+                       ef = NULL;
+                       if (_e_config_profile) break;
+                    }
+               }
+             if (!_e_config_profile)
+               {
+                  /* use system if no user profile config */
+                  e_prefix_data_concat_static(buf, "data/config/profile.cfg");
+                  ef = eet_open(buf, EET_FILE_MODE_READ);
+               }
 	  }
-	if (ef)
-	  {
-	     /* profile config exists */
-	     char *data;
-	     int data_len = 0;
-
-	     data = eet_read(ef, "config", &data_len);
-	     if ((data) && (data_len > 0))
-	       {
-		  _e_config_profile = malloc(data_len + 1);
-		  if (_e_config_profile)
-		    {
-		       memcpy(_e_config_profile, data, data_len);
-		       _e_config_profile[data_len] = 0;
-		    }
-		  free(data);
-	       }
-	     eet_close(ef);
-	  }
-	else
+        if (ef)
+          {
+             _e_config_profile = _e_config_profile_name_get(ef);
+             eet_close(ef);
+             ef = NULL;
+          }
+        if (!_e_config_profile)
 	  {
 	     /* no profile config - try other means */
 	     char *link = NULL;
@@ -99,7 +151,7 @@ e_config_init(void)
 	     /* check symlink - if default is a symlink to another dir */
 	     e_prefix_data_concat_static(buf, "data/config/default");
 	     link = ecore_file_readlink(buf);
-	     /* if so use just the filename as the priofle - must be a local link */
+	     /* if so use just the filename as the profile - must be a local link */
 	     if (link)
 	       {
 		  _e_config_profile = strdup(ecore_file_file_get(link));
@@ -108,7 +160,8 @@ e_config_init(void)
 	     else
 	       _e_config_profile = strdup("default");
 	  }
-	e_util_env_set("E_CONF_PROFILE", _e_config_profile);
+	if (!getenv("E_CONF_PROFILE"))
+	  e_util_env_set("E_CONF_PROFILE", _e_config_profile);
      }
 
    _e_config_gadcon_client_edd = E_CONFIG_DD_NEW("E_Config_Gadcon_Client", E_Config_Gadcon_Client);
@@ -139,6 +192,7 @@ e_config_init(void)
 #define D _e_config_gadcon_edd
    E_CONFIG_VAL(D, T, name, STR);
    E_CONFIG_VAL(D, T, id, INT);
+   E_CONFIG_VAL(D, T, zone, INT);
    E_CONFIG_LIST(D, T, clients, _e_config_gadcon_client_edd);
 
    _e_config_shelf_desk_edd = E_CONFIG_DD_NEW("E_Config_Shelf_Desk", E_Config_Shelf_Desk);
@@ -306,6 +360,18 @@ e_config_init(void)
    E_CONFIG_VAL(D, T, action, STR);
    E_CONFIG_VAL(D, T, params, STR);
 
+   _e_config_bindings_acpi_edd = E_CONFIG_DD_NEW("E_Config_Binding_Acpi", 
+						 E_Config_Binding_Acpi);
+#undef T
+#undef D
+#define T E_Config_Binding_Acpi
+#define D _e_config_bindings_acpi_edd
+   E_CONFIG_VAL(D, T, context, INT);
+   E_CONFIG_VAL(D, T, type, INT);
+   E_CONFIG_VAL(D, T, status, INT);
+   E_CONFIG_VAL(D, T, action, STR);
+   E_CONFIG_VAL(D, T, params, STR);
+
    _e_config_remember_edd = E_CONFIG_DD_NEW("E_Remember", E_Remember);
 #undef T
 #undef D
@@ -313,6 +379,7 @@ e_config_init(void)
 #define D _e_config_remember_edd
    E_CONFIG_VAL(D, T, match, INT);
    E_CONFIG_VAL(D, T, apply_first_only, UCHAR);
+   E_CONFIG_VAL(D, T, keep_settings, UCHAR);
    E_CONFIG_VAL(D, T, name, STR);
    E_CONFIG_VAL(D, T, class, STR);
    E_CONFIG_VAL(D, T, title, STR);
@@ -366,7 +433,8 @@ e_config_init(void)
    E_CONFIG_VAL(D, T, prop.head, INT);
    E_CONFIG_VAL(D, T, prop.command, STR);
    E_CONFIG_VAL(D, T, prop.icon_preference, UCHAR);
-   
+   E_CONFIG_VAL(D, T, prop.desktop_file, STR);
+
    _e_config_color_class_edd = E_CONFIG_DD_NEW("E_Color_Class", E_Color_Class);
 #undef T
 #undef D
@@ -458,6 +526,7 @@ e_config_init(void)
    E_CONFIG_LIST(D, T, edge_bindings, _e_config_bindings_edge_edd); /**/
    E_CONFIG_LIST(D, T, signal_bindings, _e_config_bindings_signal_edd); /**/
    E_CONFIG_LIST(D, T, wheel_bindings, _e_config_bindings_wheel_edd); /**/
+   E_CONFIG_LIST(D, T, acpi_bindings, _e_config_bindings_acpi_edd); /**/
    E_CONFIG_LIST(D, T, path_append_data, _e_config_path_append_edd); /**/
    E_CONFIG_LIST(D, T, path_append_images, _e_config_path_append_edd); /**/
    E_CONFIG_LIST(D, T, path_append_fonts, _e_config_path_append_edd); /**/
@@ -539,19 +608,6 @@ e_config_init(void)
    E_CONFIG_VAL(D, T, fullscreen_policy, INT); /**/
    E_CONFIG_VAL(D, T, input_method, STR); /**/
    E_CONFIG_LIST(D, T, path_append_messages, _e_config_path_append_edd); /**/
-   E_CONFIG_VAL(D, T, exebuf_max_exe_list, INT);
-   E_CONFIG_VAL(D, T, exebuf_max_eap_list, INT);
-   E_CONFIG_VAL(D, T, exebuf_max_hist_list, INT);
-   E_CONFIG_VAL(D, T, exebuf_scroll_animate, INT);
-   E_CONFIG_VAL(D, T, exebuf_scroll_speed, DOUBLE);
-   E_CONFIG_VAL(D, T, exebuf_pos_align_x, DOUBLE);
-   E_CONFIG_VAL(D, T, exebuf_pos_align_y, DOUBLE);
-   E_CONFIG_VAL(D, T, exebuf_pos_size_w, DOUBLE);
-   E_CONFIG_VAL(D, T, exebuf_pos_size_h, DOUBLE);
-   E_CONFIG_VAL(D, T, exebuf_pos_min_w, INT);
-   E_CONFIG_VAL(D, T, exebuf_pos_min_h, INT);
-   E_CONFIG_VAL(D, T, exebuf_pos_max_w, INT);
-   E_CONFIG_VAL(D, T, exebuf_pos_max_h, INT);
    E_CONFIG_VAL(D, T, exebuf_term_cmd, STR);
    E_CONFIG_LIST(D, T, color_classes, _e_config_color_class_edd);
    E_CONFIG_VAL(D, T, use_app_icon, INT);
@@ -566,6 +622,7 @@ e_config_init(void)
    E_CONFIG_VAL(D, T, desklock_auth_method, INT);
    E_CONFIG_VAL(D, T, desklock_login_box_zone, INT);
    E_CONFIG_VAL(D, T, desklock_start_locked, INT);
+   E_CONFIG_VAL(D, T, desklock_on_suspend, INT);
    E_CONFIG_VAL(D, T, desklock_autolock_screensaver, INT);
    E_CONFIG_VAL(D, T, desklock_post_screensaver_time, DOUBLE);
    E_CONFIG_VAL(D, T, desklock_autolock_idle, INT);
@@ -655,9 +712,9 @@ e_config_init(void)
    E_CONFIG_VAL(D, T, thumbscroll_momentum_threshhold, DOUBLE);
    E_CONFIG_VAL(D, T, thumbscroll_friction, DOUBLE);
    
-   E_CONFIG_VAL(D, T, hal_desktop, INT);
-   E_CONFIG_VAL(D, T, hal_auto_mount, INT);
-   E_CONFIG_VAL(D, T, hal_auto_open, INT);
+   E_CONFIG_VAL(D, T, dbus_desktop, INT);
+   E_CONFIG_VAL(D, T, dbus_auto_mount, INT);
+   E_CONFIG_VAL(D, T, dbus_auto_open, INT);
 
    E_CONFIG_VAL(D, T, border_keyboard.timeout, DOUBLE);
    E_CONFIG_VAL(D, T, border_keyboard.move.dx, UCHAR);
@@ -689,6 +746,10 @@ e_config_init(void)
    E_CONFIG_VAL(D, T, mode.presentation, UCHAR);
    E_CONFIG_VAL(D, T, mode.offline, UCHAR);
 
+   E_CONFIG_VAL(D, T, exec.expire_timeout, DOUBLE);
+   E_CONFIG_VAL(D, T, exec.show_run_dialog, UCHAR);
+   E_CONFIG_VAL(D, T, exec.show_exit_dialog, UCHAR);
+   
    e_config_load();
    
    e_config_save_queue();
@@ -709,6 +770,7 @@ e_config_shutdown(void)
    E_CONFIG_DD_FREE(_e_config_bindings_edge_edd);
    E_CONFIG_DD_FREE(_e_config_bindings_signal_edd);
    E_CONFIG_DD_FREE(_e_config_bindings_wheel_edd);
+   E_CONFIG_DD_FREE(_e_config_bindings_acpi_edd);
    E_CONFIG_DD_FREE(_e_config_path_append_edd);
    E_CONFIG_DD_FREE(_e_config_desktop_bg_edd);
    E_CONFIG_DD_FREE(_e_config_desktop_name_edd);
@@ -792,6 +854,15 @@ e_config_load(void)
      {
         printf("EEEK! no config of any sort! abort abort abort!\n");
         fprintf(stderr, "EEEK! no config of any sort! abort abort abort!\n");
+        e_alert_show("Enlightenment was started without any configuration\n"
+                     "files available for the given profile (normally\n"
+                     "default or the last profile used or provided on the\n"
+                     "command-line with -profile etc.)\n"
+                     "\n"
+                     "Cannot contiue without configuration to work with.\n"
+                     "Please ensure you have system or user configuration\n"
+                     "for the profile you are using before proceeeding."
+                     );
         abort();
      }
    if (e_config->config_version < E_CONFIG_FILE_VERSION)
@@ -911,11 +982,6 @@ e_config_load(void)
 	COPYVAL(desklock_ask_presentation_timeout);
 	COPYVAL(screensaver_ask_presentation);
 	COPYVAL(screensaver_ask_presentation_timeout);
-	IFCFGELSE;
-	e_config->desklock_ask_presentation = 1;
-	e_config->desklock_ask_presentation_timeout = 30.0;
-	e_config->screensaver_ask_presentation = 1;
-	e_config->screensaver_ask_presentation_timeout = 30.0;
 	IFCFGEND;
 
         IFCFG(0x0133);
@@ -924,6 +990,20 @@ e_config_load(void)
         COPYVAL(desk_flip_pan_y_axis_factor);
         IFCFGEND;
 
+        IFCFG(0x0134);
+        COPYVAL(exec.expire_timeout);
+        COPYVAL(exec.show_run_dialog);
+        COPYVAL(exec.show_exit_dialog);
+        IFCFGEND;
+
+	IFCFG(0x0136);
+	_e_config_acpi_bindings_add();
+	IFCFGEND;
+
+        IFCFG(0x0137);
+        COPYVAL(desklock_on_suspend);
+        IFCFGEND;
+        
         e_config->config_version = E_CONFIG_FILE_VERSION;   
         _e_config_free(tcfg);
      }
@@ -1029,7 +1109,7 @@ e_config_load(void)
    E_CONFIG_LIMIT(e_config->desk_flip_pan_bg, 0, 1);
    E_CONFIG_LIMIT(e_config->desk_flip_pan_x_axis_factor, 0.0, 1.0);
    E_CONFIG_LIMIT(e_config->desk_flip_pan_y_axis_factor, 0.0, 1.0);
-   E_CONFIG_LIMIT(e_config->remember_internal_windows, 0, 1);
+   E_CONFIG_LIMIT(e_config->remember_internal_windows, 0, 3);
    E_CONFIG_LIMIT(e_config->desk_auto_switch, 0, 1);
 
    E_CONFIG_LIMIT(e_config->dpms_enable, 0, 1);
@@ -1069,6 +1149,10 @@ e_config_load(void)
    E_CONFIG_LIMIT(e_config->mode.presentation, 0, 1);
    E_CONFIG_LIMIT(e_config->mode.offline, 0, 1);
 
+   E_CONFIG_LIMIT(e_config->exec.expire_timeout, 0.1, 1000);
+   E_CONFIG_LIMIT(e_config->exec.show_run_dialog, 0, 1);
+   E_CONFIG_LIMIT(e_config->exec.show_exit_dialog, 0, 1);
+   
    /* FIXME: disabled auto apply because it causes problems */
    e_config->cfgdlg_auto_apply = 0;
    /* FIXME: desklock personalized password id disabled for security reasons */
@@ -1167,7 +1251,7 @@ e_config_profile_list(void)
 	files = eina_list_sort(files, 0, (Eina_Compare_Cb)_cb_sort_files);
 	EINA_LIST_FREE(files, file)
 	  {
-	     if (ecore_strlcpy(p, file, len) >= len)
+	     if (eina_strlcpy(p, file, len) >= len)
 	       {
 		  free(file);
 		  continue;
@@ -1195,7 +1279,7 @@ e_config_profile_list(void)
 	files = eina_list_sort(files, 0, (Eina_Compare_Cb)_cb_sort_files);
 	EINA_LIST_FREE(files, file)
 	  {
-	     if (ecore_strlcpy(p, file, len) >= len)
+	     if (eina_strlcpy(p, file, len) >= len)
 	       {
 		  free(file);
 		  continue;
@@ -1274,6 +1358,7 @@ e_config_domain_load(const char *domain, E_Config_DD *edd)
    Eet_File *ef;
    char buf[4096];
    void *data = NULL;
+   int i;
 
    e_user_dir_snprintf(buf, sizeof(buf), "config/%s/%s.cfg",
 		       _e_config_profile, domain);
@@ -1282,7 +1367,20 @@ e_config_domain_load(const char *domain, E_Config_DD *edd)
      {
 	data = eet_data_read(ef, edd, "config");
 	eet_close(ef);
-        return data;
+        if (data) return data;
+     }
+   
+   for (i =1; i <= _e_config_revisions; i++)
+     {
+        e_user_dir_snprintf(buf, sizeof(buf), "config/%s/%s.%i.cfg",
+                            _e_config_profile, domain, i);
+        ef = eet_open(buf, EET_FILE_MODE_READ);
+        if (ef)
+          {
+             data = eet_data_read(ef, edd, "config");
+             eet_close(ef);
+             if (data) return data;
+          }
      }
    return e_config_domain_system_load(domain, edd);
 }
@@ -1326,8 +1424,23 @@ e_config_profile_save(void)
 	if (_e_config_eet_close_handle(ef, buf2))
 	  {
 	     int ret;
-
-	     ret = ecore_file_mv(buf2, buf);
+             
+             if (_e_config_revisions > 0)
+               {
+                  int i;
+                  char bsrc[4096], bdst[4096];
+                  
+                  for (i = _e_config_revisions; i > 1; i--)
+                    {
+                       e_user_dir_snprintf(bsrc, sizeof(bsrc), "config/profile.%i.cfg", i - 1);
+                       e_user_dir_snprintf(bdst, sizeof(bdst), "config/profile.%i.cfg", i);
+                       ecore_file_mv(bsrc, bdst);
+                    }
+                  e_user_dir_snprintf(bsrc, sizeof(bsrc), "config/profile.cfg");
+                  e_user_dir_snprintf(bdst, sizeof(bdst), "config/profile.1.cfg");
+                  ecore_file_mv(bsrc, bdst);
+               }
+             ret = ecore_file_mv(buf2, buf);
 	     if (!ret)
 	       {
 		  printf("*** Error saving profile. ***");
@@ -1356,7 +1469,7 @@ e_config_domain_save(const char *domain, E_Config_DD *edd, const void *data)
    buf[len] = '/';
    len++;
 
-   len2 = ecore_strlcpy(buf + len, domain, sizeof(buf) - len);
+   len2 = eina_strlcpy(buf + len, domain, sizeof(buf) - len);
    if (len2 + sizeof(".cfg") >= sizeof(buf) - len) return 0;
 
    len += len2;
@@ -1374,10 +1487,25 @@ e_config_domain_save(const char *domain, E_Config_DD *edd, const void *data)
 	ok = eet_data_write(ef, edd, "config", data, 1);
 	if (_e_config_eet_close_handle(ef, buf2))
 	  {
+             if (_e_config_revisions > 0)
+               {
+                  int i;
+                  char bsrc[4096], bdst[4096];
+                  
+                  for (i = _e_config_revisions; i > 1; i--)
+                    {
+                       e_user_dir_snprintf(bsrc, sizeof(bsrc), "config/%s/%s.%i.cfg", _e_config_profile, domain, i - 1);
+                       e_user_dir_snprintf(bdst, sizeof(bdst), "config/%s/%s.%i.cfg", _e_config_profile, domain, i);
+                       ecore_file_mv(bsrc, bdst);
+                    }
+                  e_user_dir_snprintf(bsrc, sizeof(bsrc), "config/%s/%s.cfg", _e_config_profile, domain);
+                  e_user_dir_snprintf(bdst, sizeof(bdst), "config/%s/%s.1.cfg", _e_config_profile, domain);
+                  ecore_file_mv(bsrc, bdst);
+               }
 	     ret = ecore_file_mv(buf2, buf);
 	     if (!ret)
 	       {
-		  printf("*** Error saving profile. ***");
+		  printf("*** Error saving config. ***");
 	       }
 	  }
 	ecore_file_unlink(buf2);
@@ -1389,12 +1517,10 @@ EAPI E_Config_Binding_Mouse *
 e_config_binding_mouse_match(E_Config_Binding_Mouse *eb_in)
 {
    Eina_List *l;
-   
-   for (l = e_config->mouse_bindings; l; l = l->next)
+   E_Config_Binding_Mouse *eb;
+
+   EINA_LIST_FOREACH(e_config->mouse_bindings, l, eb)   
      {
-	E_Config_Binding_Mouse *eb;
-	
-	eb = l->data;
 	if ((eb->context == eb_in->context) &&
 	    (eb->button == eb_in->button) &&
 	    (eb->modifiers == eb_in->modifiers) &&
@@ -1412,12 +1538,10 @@ EAPI E_Config_Binding_Key *
 e_config_binding_key_match(E_Config_Binding_Key *eb_in)
 {
    Eina_List *l;
+   E_Config_Binding_Key *eb;
    
-   for (l = e_config->key_bindings; l; l = l->next)
+   EINA_LIST_FOREACH(e_config->mouse_bindings, l, eb)   
      {
-	E_Config_Binding_Key *eb;
-	
-	eb = l->data;
 	if ((eb->context == eb_in->context) &&
 	    (eb->modifiers == eb_in->modifiers) &&
 	    (eb->any_mod == eb_in->any_mod) &&
@@ -1436,12 +1560,10 @@ EAPI E_Config_Binding_Edge *
 e_config_binding_edge_match(E_Config_Binding_Edge *eb_in)
 {
    Eina_List *l;
+   E_Config_Binding_Edge *eb;
    
-   for (l = e_config->edge_bindings; l; l = l->next)
+   EINA_LIST_FOREACH(e_config->edge_bindings, l, eb)
      {
-	E_Config_Binding_Edge *eb;
-	
-	eb = l->data;
 	if ((eb->context == eb_in->context) &&
 	    (eb->modifiers == eb_in->modifiers) &&
 	    (eb->any_mod == eb_in->any_mod) &&
@@ -1460,12 +1582,10 @@ EAPI E_Config_Binding_Signal *
 e_config_binding_signal_match(E_Config_Binding_Signal *eb_in)
 {
    Eina_List *l;
+   E_Config_Binding_Signal *eb;
    
-   for (l = e_config->signal_bindings; l; l = l->next)
+   EINA_LIST_FOREACH(e_config->signal_bindings, l, eb)
      {
-	E_Config_Binding_Signal *eb;
-	
-	eb = l->data;
 	if ((eb->context == eb_in->context) &&
 	    (eb->modifiers == eb_in->modifiers) &&
 	    (eb->any_mod == eb_in->any_mod) &&
@@ -1486,21 +1606,41 @@ EAPI E_Config_Binding_Wheel *
 e_config_binding_wheel_match(E_Config_Binding_Wheel *eb_in)
 {
    Eina_List *l;
-   
-   for (l = e_config->wheel_bindings; l; l = l->next)
+   E_Config_Binding_Wheel *eb;
+
+   EINA_LIST_FOREACH(e_config->wheel_bindings, l, eb)
      {
-	E_Config_Binding_Wheel *eb;
-	
-	eb = l->data;
 	if ((eb->context == eb_in->context) &&
 	    (eb->direction == eb_in->direction) &&
 	    (eb->z == eb_in->z) &&
 	    (eb->modifiers == eb_in->modifiers) &&
 	    (eb->any_mod == eb_in->any_mod) &&
 	    (((eb->action) && (eb_in->action) && (!strcmp(eb->action, eb_in->action))) ||
-	     ((!eb->action) && (!eb_in->action))) &&
+		((!eb->action) && (!eb_in->action))) &&
 	    (((eb->params) && (eb_in->params) && (!strcmp(eb->params, eb_in->params))) ||
-	     ((!eb->params) && (!eb_in->params))))
+		((!eb->params) && (!eb_in->params))))
+	  return eb;
+     }
+   return NULL;
+}
+
+EAPI E_Config_Binding_Acpi *
+e_config_binding_acpi_match(E_Config_Binding_Acpi *eb_in) 
+{
+   Eina_List *l;
+   E_Config_Binding_Acpi *eb;
+
+   EINA_LIST_FOREACH(e_config->acpi_bindings, l, eb)
+     {
+	if ((eb->context == eb_in->context) &&
+	    (eb->type == eb_in->type) && 
+	    (eb->status == eb_in->status) && 
+	    (((eb->action) && (eb_in->action) && 
+	      (!strcmp(eb->action, eb_in->action))) ||
+		((!eb->action) && (!eb_in->action))) &&
+	    (((eb->params) && (eb_in->params) && 
+	      (!strcmp(eb->params, eb_in->params))) ||
+		((!eb->params) && (!eb_in->params))))
 	  return eb;
      }
    return NULL;
@@ -1531,6 +1671,7 @@ _e_config_free(E_Config *ecf)
    E_Config_Syscon_Action *sca;
    E_Config_Binding_Key *ebk;
    E_Config_Binding_Edge *ebe;
+   E_Config_Binding_Acpi *eba;
    E_Font_Fallback *eff;
    E_Config_Module *em;
    E_Font_Default *efd;
@@ -1596,6 +1737,12 @@ _e_config_free(E_Config *ecf)
         if (ebw->params) eina_stringshare_del(ebw->params);
         E_FREE(ebw);
      }
+   EINA_LIST_FREE(ecf->acpi_bindings, eba)
+     {
+        if (eba->action) eina_stringshare_del(eba->action);
+        if (eba->params) eina_stringshare_del(eba->params);
+        E_FREE(eba);
+     }
    EINA_LIST_FREE(ecf->path_append_data, epd)
      {
         if (epd->dir) eina_stringshare_del(epd->dir);
@@ -1683,7 +1830,7 @@ _e_config_free(E_Config *ecf)
    E_FREE(ecf);
 }
 
-static int
+static Eina_Bool
 _e_config_cb_timer(void *data)
 {
    e_util_dialog_show(_("Settings Upgraded"), "%s", (char *)data);
@@ -1797,3 +1944,49 @@ _e_config_eet_close_handle(Eet_File *ef, char *file)
      }
    return 1;
 }
+
+static void 
+_e_config_acpi_bindings_add(void) 
+{
+   E_Config_Binding_Acpi *bind;
+
+   bind = E_NEW(E_Config_Binding_Acpi, 1);
+   bind->context = E_BINDING_CONTEXT_NONE;
+   bind->type = E_ACPI_TYPE_AC_ADAPTER;
+   bind->status = 0;
+   bind->action = eina_stringshare_add("dim_screen");
+   bind->params = NULL;
+   e_config->acpi_bindings = eina_list_append(e_config->acpi_bindings, bind);
+
+   bind = E_NEW(E_Config_Binding_Acpi, 1);
+   bind->context = E_BINDING_CONTEXT_NONE;
+   bind->type = E_ACPI_TYPE_AC_ADAPTER;
+   bind->status = 1;
+   bind->action = eina_stringshare_add("undim_screen");
+   bind->params = NULL;
+   e_config->acpi_bindings = eina_list_append(e_config->acpi_bindings, bind);
+
+   bind = E_NEW(E_Config_Binding_Acpi, 1);
+   bind->context = E_BINDING_CONTEXT_NONE;
+   bind->type = E_ACPI_TYPE_LID;
+   bind->status = 0;
+   bind->action = eina_stringshare_add("suspend");
+   bind->params = eina_stringshare_add("now");
+   e_config->acpi_bindings = eina_list_append(e_config->acpi_bindings, bind);
+
+   bind = E_NEW(E_Config_Binding_Acpi, 1);
+   bind->context = E_BINDING_CONTEXT_NONE;
+   bind->type = E_ACPI_TYPE_POWER;
+   bind->status = -1;
+   bind->action = eina_stringshare_add("halt_now");
+   bind->params = eina_stringshare_add("now");
+   e_config->acpi_bindings = eina_list_append(e_config->acpi_bindings, bind);
+
+   bind = E_NEW(E_Config_Binding_Acpi, 1);
+   bind->context = E_BINDING_CONTEXT_NONE;
+   bind->type = E_ACPI_TYPE_SLEEP;
+   bind->status = -1;
+   bind->action = eina_stringshare_add("suspend");
+   bind->params = eina_stringshare_add("now");
+   e_config->acpi_bindings = eina_list_append(e_config->acpi_bindings, bind);
+}
