@@ -12,20 +12,20 @@
  * - Launch .desktop in terminal if .desktop requires it
  */
 
-typedef struct _E_Exec_Launch   E_Exec_Launch;
-typedef struct _E_Exec_Search   E_Exec_Search;
+typedef struct _E_Exec_Launch E_Exec_Launch;
+typedef struct _E_Exec_Search E_Exec_Search;
 
 struct _E_Exec_Launch
 {
-   E_Zone         *zone;
-   const char     *launch_method;
+   E_Zone *zone;
+   const char *launch_method;
 };
 
 struct _E_Exec_Search
 {
    Efreet_Desktop *desktop;
-   int             startup_id;
-   pid_t           pid;
+   int startup_id;
+   pid_t pid;
 };
 
 struct _E_Config_Dialog_Data
@@ -33,35 +33,34 @@ struct _E_Config_Dialog_Data
    Efreet_Desktop *desktop;
    char *exec;
 
-   Ecore_Exe_Event_Del   event;
+   Ecore_Exe_Event_Del event;
    Ecore_Exe_Event_Data *error;
    Ecore_Exe_Event_Data *read;
 
-   char *label;
-   char *exit;
-   char *signal;
+   char *label, *exit, *signal;
 };
 
 /* local subsystem functions */
 static E_Exec_Instance *_e_exec_cb_exec(void *data, Efreet_Desktop *desktop, char *exec, int remaining);
-static int  _e_exec_cb_expire_timer(void *data);
-static int  _e_exec_cb_exit(void *data, int type, void *event);
+static Eina_Bool _e_exec_cb_expire_timer(void *data);
+static Eina_Bool _e_exec_cb_exit(void *data, int type, void *event);
 
-static Evas_Bool _e_exec_startup_id_pid_find(const Eina_Hash *hash __UNUSED__, const void *key __UNUSED__, void *value, void *data);
+static Eina_Bool _e_exec_startup_id_pid_find(const Eina_Hash *hash __UNUSED__, const void *key __UNUSED__, void *value, void *data);
 
-static void         _e_exec_error_dialog(Efreet_Desktop *desktop, const char *exec, Ecore_Exe_Event_Del *event, Ecore_Exe_Event_Data *error, Ecore_Exe_Event_Data *read);
-static void         _fill_data(E_Config_Dialog_Data *cfdata);
-static void        *_create_data(E_Config_Dialog *cfd);
-static void         _free_data(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata);
+static void _e_exec_error_dialog(Efreet_Desktop *desktop, const char *exec, Ecore_Exe_Event_Del *event, Ecore_Exe_Event_Data *error, Ecore_Exe_Event_Data *read);
+static void _fill_data(E_Config_Dialog_Data *cfdata);
+static void *_create_data(E_Config_Dialog *cfd);
+static void _free_data(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata);
 static Evas_Object *_basic_create_widgets(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cfdata);
 static Evas_Object *_advanced_create_widgets(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cfdata);
 static Evas_Object *_dialog_scrolltext_create(Evas *evas, char *title, Ecore_Exe_Event_Data_Line *lines);
-static void         _dialog_save_cb(void *data, void *data2);
+static void _dialog_save_cb(void *data, void *data2);
+static void _e_exec_instance_free(E_Exec_Instance *inst);
 
 /* local subsystem globals */
-static Eina_List   *e_exec_start_pending = NULL;
-static Eina_Hash   *e_exec_instances = NULL;
-static int          startup_id = 0;
+static Eina_List *e_exec_start_pending = NULL;
+static Eina_Hash *e_exec_instances = NULL;
+static int startup_id = 0;
 
 static Ecore_Event_Handler *_e_exec_exit_handler = NULL;
 static Ecore_Event_Handler *_e_exec_border_add_handler = NULL;
@@ -90,7 +89,8 @@ e_exec_shutdown(void)
    e_util_env_set("E_STARTUP_ID", buf);
 
    if (_e_exec_exit_handler) ecore_event_handler_del(_e_exec_exit_handler);
-   if (_e_exec_border_add_handler) ecore_event_handler_del(_e_exec_border_add_handler);
+   if (_e_exec_border_add_handler) 
+     ecore_event_handler_del(_e_exec_border_add_handler);
    eina_hash_free(e_exec_instances);
    eina_list_free(e_exec_start_pending);
    return 1;
@@ -113,13 +113,14 @@ e_exec(E_Zone *zone, Efreet_Desktop *desktop, const char *exec,
      }
    if (launch_method) 
      launch->launch_method = eina_stringshare_add(launch_method);
-
+   
    if (desktop)
      {
 	if (exec)
 	  inst = _e_exec_cb_exec(launch, NULL, strdup(exec), 0);
-	else
-	  inst = efreet_desktop_command_get(desktop, files, _e_exec_cb_exec, launch);
+	else 
+          inst = efreet_desktop_command_get(desktop, files, 
+                                            (Efreet_Desktop_Command_Cb) _e_exec_cb_exec, launch);
      }
    else
      inst = _e_exec_cb_exec(launch, NULL, strdup(exec), 0);
@@ -158,6 +159,7 @@ _e_exec_cb_exec(void *data, Efreet_Desktop *desktop, char *exec, int remaining)
    if (startup_id == 0)
      {
 	const char *p;
+
 	p = getenv("E_STARTUP_ID");
 	if (p) startup_id = atoi(p);
 	e_util_env_set("E_STARTUP_ID", NULL);
@@ -200,6 +202,10 @@ _e_exec_cb_exec(void *data, Efreet_Desktop *desktop, char *exec, int remaining)
    snprintf(buf, sizeof(buf), "E_START|%i", startup_id);
    e_util_env_set("DESKTOP_STARTUP_ID", buf);
 
+   // dont set vsync for clients - maybe inherited from compositore. fixme:
+   // need a way to still inherit from parent env of wm.
+   e_util_env_set("__GL_SYNC_TO_VBLANK", NULL);
+   
    e_util_library_path_strip();
 //// FIXME: seem to be some issues with the pipe and filling up ram - need to
 //// check. for now disable.   
@@ -225,7 +231,8 @@ _e_exec_cb_exec(void *data, Efreet_Desktop *desktop, char *exec, int remaining)
 	return NULL;
      }
    /* reset env vars */
-   if (launch->launch_method) e_exehist_add(launch->launch_method, exec);
+   if (launch->launch_method && !desktop) 
+     e_exehist_add(launch->launch_method, exec);
    free(exec);
    /* 20 lines at start and end, 20x100 limit on bytes at each end. */
 //// FIXME: seem to be some issues with the pipe and filling up ram - need to
@@ -236,26 +243,25 @@ _e_exec_cb_exec(void *data, Efreet_Desktop *desktop, char *exec, int remaining)
    if (desktop)
      {
 	Eina_List *l;
-
+        
 	efreet_desktop_ref(desktop);
 	inst->desktop = desktop;
+        inst->key = eina_stringshare_add(desktop->orig_path);
 	inst->exe = exe;
 	inst->startup_id = startup_id;
 	inst->launch_time = ecore_time_get();
-	inst->expire_timer = ecore_timer_add(10.0, _e_exec_cb_expire_timer, inst);
-
+	inst->expire_timer = ecore_timer_add(e_config->exec.expire_timeout, 
+                                             _e_exec_cb_expire_timer, inst);
+        
 	l = eina_hash_find(e_exec_instances, desktop->orig_path);
+        l = eina_list_append(l, inst);
 	if (l)
-	  {
-	     l = eina_list_append(l, inst);
-	     eina_hash_modify(e_exec_instances, desktop->orig_path, l);
-	  }
+          eina_hash_modify(e_exec_instances, desktop->orig_path, l);
 	else
-	  {
-	     l = eina_list_append(l, inst);
-	     eina_hash_add(e_exec_instances, desktop->orig_path, l);
-	  }
+          eina_hash_add(e_exec_instances, desktop->orig_path, l);
 	e_exec_start_pending = eina_list_append(e_exec_start_pending, desktop);
+        
+	e_exehist_add(launch->launch_method, desktop->exec);
      }
    else if (exe)
      {
@@ -268,102 +274,136 @@ _e_exec_cb_exec(void *data, Efreet_Desktop *desktop, char *exec, int remaining)
      {
 	if (launch->launch_method) eina_stringshare_del(launch->launch_method);
 	if (launch->zone) e_object_unref(E_OBJECT(launch->zone));
-      	free(launch);
+        free(launch);
      }
    return inst;
 }
 
-static int
+static Eina_Bool
 _e_exec_cb_expire_timer(void *data)
 {
    E_Exec_Instance *inst;
- 
+
    inst = data;
    e_exec_start_pending = eina_list_remove(e_exec_start_pending, inst->desktop);
    inst->expire_timer = NULL;
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
-static int
-_e_exec_cb_exit(void *data, int type, void *event)
+static void
+_e_exec_instance_free(E_Exec_Instance *inst)
 {
    Eina_List *instances;
+   
+   if (inst->key)
+     {
+	instances = eina_hash_find(e_exec_instances, inst->key);
+	if (instances)
+	  {
+	     instances = eina_list_remove(instances, inst);
+	     if (instances)
+	       eina_hash_modify(e_exec_instances, inst->key, instances);
+	     else
+	       eina_hash_del(e_exec_instances, inst->key, NULL);
+	  }
+        eina_stringshare_del(inst->key);
+     }
+   e_exec_start_pending = eina_list_remove(e_exec_start_pending, inst->desktop);
+   if (inst->expire_timer) ecore_timer_del(inst->expire_timer);
+   if (inst->desktop) efreet_desktop_free(inst->desktop);
+   free(inst); 
+}
+
+
+
+static Eina_Bool
+_e_exec_cb_instance_finish(void *data)
+{
+   _e_exec_instance_free(data);
+   return ECORE_CALLBACK_CANCEL;
+}
+
+
+static Eina_Bool
+_e_exec_cb_exit(__UNUSED__ void *data, __UNUSED__ int type, void *event)
+{
    Ecore_Exe_Event_Del *ev;
    E_Exec_Instance *inst;
 
    ev = event;
-   if (!ev->exe) return 1;
+   if (!ev->exe) return ECORE_CALLBACK_PASS_ON;
 //   if (ecore_exe_tag_get(ev->exe)) printf("  tag %s\n", ecore_exe_tag_get(ev->exe));
-   if (!(ecore_exe_tag_get(ev->exe) && 
-	 (!strcmp(ecore_exe_tag_get(ev->exe), "E/exec")))) return 1;
+   if (!(ecore_exe_tag_get(ev->exe) &&
+	 (!strcmp(ecore_exe_tag_get(ev->exe), "E/exec"))))
+     return ECORE_CALLBACK_PASS_ON;
    inst = ecore_exe_data_get(ev->exe);
-   if (!inst) return 1;
+   if (!inst) return ECORE_CALLBACK_PASS_ON;
 
    /* /bin/sh uses this if cmd not found */
    if ((ev->exited) &&
        ((ev->exit_code == 127) || (ev->exit_code == 255)))
      {
-	E_Dialog *dia;
-	
-	dia = e_dialog_new(e_container_current_get(e_manager_current_get()),
-			   "E", "_e_exec_run_error_dialog");
-	if (dia)
-	  {
-	     char buf[4096];
-	     
-	     e_dialog_title_set(dia, _("Application run error"));
-	     snprintf(buf, sizeof(buf),
-		      _("Enlightenment was unable to run the application:<br>"
-			"<br>"
-			"%s<br>"
-			"<br>"
-			"The application failed to start."),
-		      ecore_exe_cmd_get(ev->exe));
-	     e_dialog_text_set(dia, buf);
-	     e_dialog_button_add(dia, _("OK"), NULL, NULL, NULL);
-	     e_dialog_button_focus_num(dia, 1);
-	     e_win_centered_set(dia->win, 1);
-	     e_dialog_show(dia);
-	  }
+        if (e_config->exec.show_run_dialog)
+          {
+             E_Dialog *dia;
+             
+             dia = e_dialog_new(e_container_current_get(e_manager_current_get()),
+                                "E", "_e_exec_run_error_dialog");
+             if (dia)
+               {
+                  char buf[4096];
+                  
+                  e_dialog_title_set(dia, _("Application run error"));
+                  snprintf(buf, sizeof(buf),
+                           _("Enlightenment was unable to run the application:<br>"
+                             "<br>"
+                             "%s<br>"
+                             "<br>"
+                             "The application failed to start."),
+                           ecore_exe_cmd_get(ev->exe));
+                  e_dialog_text_set(dia, buf);
+                  e_dialog_button_add(dia, _("OK"), NULL, NULL, NULL);
+                  e_dialog_button_focus_num(dia, 1);
+                  e_win_centered_set(dia->win, 1);
+                  e_dialog_show(dia);
+               }
+          }
      }
    /* Let's hope that everything returns this properly. */
    else if (!((ev->exited) && (ev->exit_code == EXIT_SUCCESS))) 
      {
-	/* filter out common exits via signals - int/term/quit. not really
-	 * worth popping up a dialog for */
-	if (!
-	    ((ev->signalled) &&
-	     ((ev->exit_signal == SIGINT) ||
-	      (ev->exit_signal == SIGQUIT) ||
-	      (ev->exit_signal == SIGTERM)))
-	    )
-	  {
-	     /* Show the error dialog with details from the exe. */
-	     _e_exec_error_dialog(inst->desktop, ecore_exe_cmd_get(ev->exe), ev,
-				  ecore_exe_event_data_get(ev->exe, ECORE_EXE_PIPE_ERROR),
-				  ecore_exe_event_data_get(ev->exe, ECORE_EXE_PIPE_READ));
-	  }
+        if (e_config->exec.show_exit_dialog)
+          {
+             /* filter out common exits via signals - int/term/quit. not really
+              * worth popping up a dialog for */
+             if (!((ev->signalled) &&
+                   ((ev->exit_signal == SIGINT) ||
+                    (ev->exit_signal == SIGQUIT) ||
+                    (ev->exit_signal == SIGTERM)))
+                 )
+               {
+                  /* Show the error dialog with details from the exe. */
+                  _e_exec_error_dialog(inst->desktop, ecore_exe_cmd_get(ev->exe), ev,
+                                       ecore_exe_event_data_get(ev->exe, ECORE_EXE_PIPE_ERROR),
+                                       ecore_exe_event_data_get(ev->exe, ECORE_EXE_PIPE_READ));
+               }
+          }
      }
-   if (inst->desktop)
+
+   /* maybe better 1 minute? it might be openoffice */
+   if (ecore_time_get() - inst->launch_time < 2.0)
      {
-	instances = eina_hash_find(e_exec_instances, inst->desktop->orig_path);
-	if (instances)
-	  {
-	     instances = eina_list_remove(instances, inst);
-	     if (instances)
-	       eina_hash_modify(e_exec_instances, inst->desktop->orig_path, instances);
-	     else
-	       eina_hash_del(e_exec_instances, inst->desktop->orig_path, NULL);
-	  }
+        inst->exe = NULL;
+	if (inst->expire_timer) ecore_timer_del(inst->expire_timer);
+	inst->expire_timer = ecore_timer_add(e_config->exec.expire_timeout, _e_exec_cb_instance_finish, inst);
      }
-   e_exec_start_pending = eina_list_remove(e_exec_start_pending, inst->desktop);
-   if (inst->expire_timer) ecore_timer_del(inst->expire_timer);
-   if (inst->desktop) efreet_desktop_free(inst->desktop);
-   free(inst);
-   return 1;
+   else
+     _e_exec_instance_free(inst);
+
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static Evas_Bool
+static Eina_Bool
 _e_exec_startup_id_pid_find(const Eina_Hash *hash __UNUSED__, const void *key __UNUSED__, void *value, void *data)
 {
    E_Exec_Search *search;
@@ -371,24 +411,24 @@ _e_exec_startup_id_pid_find(const Eina_Hash *hash __UNUSED__, const void *key __
    Eina_List *l;
 
    search = data;
-
    EINA_LIST_FOREACH(value, l, inst)
-	if (((search->startup_id > 0) && (search->startup_id == inst->startup_id)) ||
-	    ((inst->exe) && (search->pid > 1) && 
+     {
+        if (((search->startup_id > 0) && 
+             (search->startup_id == inst->startup_id)) ||
+            ((inst->exe) && (search->pid > 1) && 
              (search->pid == ecore_exe_pid_get(inst->exe))))
-	  {
-	     search->desktop = inst->desktop;
-	     return 0;
-	  }
-
-   return 1;
+          {
+             search->desktop = inst->desktop;
+             return EINA_FALSE;
+          }
+     }
+   return EINA_TRUE;
 }
 
 static void    
 _e_exec_error_dialog(Efreet_Desktop *desktop, const char *exec, Ecore_Exe_Event_Del *event,
 		     Ecore_Exe_Event_Data *error, Ecore_Exe_Event_Data *read)
 {
-   E_Config_Dialog *cfd;
    E_Config_Dialog_View *v;
    E_Config_Dialog_Data *cfdata;
    E_Container *con;
@@ -398,7 +438,7 @@ _e_exec_error_dialog(Efreet_Desktop *desktop, const char *exec, Ecore_Exe_Event_
    cfdata = E_NEW(E_Config_Dialog_Data, 1);
    if (!cfdata)
      {
-	free(v);
+	E_FREE(v);
 	return;
      }
    cfdata->desktop = desktop;
@@ -415,9 +455,9 @@ _e_exec_error_dialog(Efreet_Desktop *desktop, const char *exec, Ecore_Exe_Event_
 
    con = e_container_current_get(e_manager_current_get());
    /* Create The Dialog */
-   cfd = e_config_dialog_new(con, _("Application Execution Error"), 
-			     "E", "_e_exec_error_exit_dialog",
-			     NULL, 0, v, cfdata);
+   e_config_dialog_new(con, _("Application Execution Error"), 
+		       "E", "_e_exec_error_exit_dialog",
+		       NULL, 0, v, cfdata);
 }
 
 static void
@@ -433,14 +473,16 @@ _fill_data(E_Config_Dialog_Data *cfdata)
    if ((cfdata->event.exited) && (!cfdata->exit))
      {
 	snprintf(buf, sizeof(buf), 
-		 _("An exit code of %i was returned from %s."), cfdata->event.exit_code, cfdata->exec);
+		 _("An exit code of %i was returned from %s."), 
+                 cfdata->event.exit_code, cfdata->exec);
 	cfdata->exit = strdup(buf);
      }
    if ((cfdata->event.signalled) && (!cfdata->signal))
      {
 	if (cfdata->event.exit_signal == SIGINT)
 	  snprintf(buf, sizeof(buf),
-		   _("%s was interrupted by an Interrupt Signal."), cfdata->desktop->exec);
+		   _("%s was interrupted by an Interrupt Signal."), 
+                   cfdata->desktop->exec);
 	else if (cfdata->event.exit_signal == SIGQUIT)
 	  snprintf(buf, sizeof(buf), _("%s was interrupted by a Quit Signal."),
 		   cfdata->exec);
@@ -449,19 +491,23 @@ _fill_data(E_Config_Dialog_Data *cfdata)
 		   _("%s was interrupted by an Abort Signal."), cfdata->exec);
 	else if (cfdata->event.exit_signal == SIGFPE)
 	  snprintf(buf, sizeof(buf),
-		   _("%s was interrupted by a Floating Point Error."), cfdata->exec);
+		   _("%s was interrupted by a Floating Point Error."), 
+                   cfdata->exec);
 	else if (cfdata->event.exit_signal == SIGKILL)
 	  snprintf(buf, sizeof(buf),
-		   _("%s was interrupted by an Uninterruptable Kill Signal."), cfdata->exec);
+		   _("%s was interrupted by an Uninterruptable Kill Signal."), 
+                   cfdata->exec);
 	else if (cfdata->event.exit_signal == SIGSEGV)
 	  snprintf(buf, sizeof(buf),
-		   _("%s was interrupted by a Segmentation Fault."), cfdata->exec);
+		   _("%s was interrupted by a Segmentation Fault."), 
+                   cfdata->exec);
 	else if (cfdata->event.exit_signal == SIGPIPE)
 	  snprintf(buf, sizeof(buf), 
 		   _("%s was interrupted by a Broken Pipe."), cfdata->exec);
 	else if (cfdata->event.exit_signal == SIGTERM)
 	  snprintf(buf, sizeof(buf),
-		   _("%s was interrupted by a Termination Signal."), cfdata->exec);
+		   _("%s was interrupted by a Termination Signal."), 
+                   cfdata->exec);
 	else if (cfdata->event.exit_signal == SIGBUS)
 	  snprintf(buf, sizeof(buf), 
 		   _("%s was interrupted by a Bus Error."), cfdata->exec);
@@ -515,18 +561,16 @@ _free_data(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata)
    E_FREE(cfdata->signal);
    E_FREE(cfdata->exit);
    E_FREE(cfdata->label);
-
-   free(cfdata);
+   E_FREE(cfdata);
 }
 
 static Evas_Object *
 _dialog_scrolltext_create(Evas *evas, char *title, Ecore_Exe_Event_Data_Line *lines)
 {
-   int i;
    Evas_Object *obj, *os;
    char *text;
    char *trunc_note = _("***The remaining output has been truncated. Save the output to view.***\n");
-   int tlen, max_lines;
+   int tlen, max_lines, i;
 
    os = e_widget_framelist_add(evas, _(title), 0);
 
@@ -559,12 +603,11 @@ _dialog_scrolltext_create(Evas *evas, char *title, Ecore_Exe_Event_Data_Line *li
 	  }
 
 	/* Append the warning about truncated output. */
-	if (lines[max_lines].line != NULL)
-	  strcat(text, trunc_note);
+	if (lines[max_lines].line != NULL) strcat(text, trunc_note);
 
 	e_widget_textblock_plain_set(obj, text);
      }
-   e_widget_min_size_set(obj, 240, 120);
+   e_widget_size_min_set(obj, 240, 120);
 
    e_widget_framelist_object_append(os, obj);
 
@@ -585,11 +628,11 @@ _basic_create_widgets(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cf
    ob = e_widget_label_add(evas, cfdata->label);
    e_widget_list_object_append(o, ob, 1, 1, 0.5);
 
-   if (cfdata->error)
-     error_length = cfdata->error->size;
+   if (cfdata->error) error_length = cfdata->error->size;
    if (error_length)
      {
-	os = _dialog_scrolltext_create(evas, _("Error Logs"), cfdata->error->lines);
+	os = _dialog_scrolltext_create(evas, _("Error Logs"), 
+                                       cfdata->error->lines);
 	e_widget_list_object_append(o, os, 1, 1, 0.5);
      }
    else
@@ -601,7 +644,7 @@ _basic_create_widgets(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cf
    ob = e_widget_button_add(evas, _("Save This Message"), "system-run", 
 			    _dialog_save_cb, NULL, cfdata);
    e_widget_list_object_append(o, ob, 0, 0, 0.5);
-   
+
    snprintf(buf, sizeof(buf), _("This error log will be saved as %s/%s.log"), 
 	    e_user_homedir_get(), cfdata->desktop->name);
    ob = e_widget_label_add(evas, buf);
@@ -642,12 +685,12 @@ _advanced_create_widgets(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data
 	e_widget_list_object_append(o, of, 1, 1, 0.5);
      }
 
-   if (cfdata->read)
-     read_length = cfdata->read->size;
+   if (cfdata->read) read_length = cfdata->read->size;
 
    if (read_length)
      {
-	of = _dialog_scrolltext_create(evas, _("Output Data"), cfdata->read->lines);
+	of = _dialog_scrolltext_create(evas, _("Output Data"), 
+                                       cfdata->read->lines);
 	/* FIXME: Add stdout "start". */
 	/* FIXME: Add stdout "end". */
      }
@@ -659,11 +702,11 @@ _advanced_create_widgets(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data
      }
    e_widget_table_object_append(ot, of, 0, 0, 1, 1, 1, 1, 1, 1);
 
-   if (cfdata->error)
-     error_length = cfdata->error->size;
+   if (cfdata->error) error_length = cfdata->error->size;
    if (error_length)
      {
-	of = _dialog_scrolltext_create(evas, _("Error Logs"), cfdata->error->lines);
+	of = _dialog_scrolltext_create(evas, _("Error Logs"), 
+                                       cfdata->error->lines);
 	/* FIXME: Add stderr "start". */
 	/* FIXME: Add stderr "end". */
      }
@@ -677,7 +720,8 @@ _advanced_create_widgets(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data
 
    e_widget_list_object_append(o, ot, 1, 1, 0.5);
 
-   ob = e_widget_button_add(evas, _("Save This Message"), "system-run", _dialog_save_cb, NULL, cfdata);
+   ob = e_widget_button_add(evas, _("Save This Message"), "system-run", 
+                            _dialog_save_cb, NULL, cfdata);
    e_widget_list_object_append(o, ob, 0, 0, 0.5);
 
    snprintf(buf, sizeof(buf), _("This error log will be saved as %s/%s.log"), 
@@ -708,17 +752,18 @@ _dialog_save_cb(void *data, void *data2)
 
    if (cfdata->exit)
      {
-	snprintf(buffer, sizeof(buffer), "Error Information:\n\t%s\n\n", cfdata->exit);
+	snprintf(buffer, sizeof(buffer), "Error Information:\n\t%s\n\n", 
+                 cfdata->exit);
 	fwrite(buffer, sizeof(char), strlen(buffer), f);
      }
    if (cfdata->signal)
      {
-	snprintf(buffer, sizeof(buffer), "Error Signal Information:\n\t%s\n\n", cfdata->signal);
+	snprintf(buffer, sizeof(buffer), "Error Signal Information:\n\t%s\n\n", 
+                 cfdata->signal);
 	fwrite(buffer, sizeof(char), strlen(buffer), f);
      }
 
-   if (cfdata->read)
-     read_length = cfdata->read->size;
+   if (cfdata->read) read_length = cfdata->read->size;
 
    if (read_length)
      {
@@ -747,8 +792,7 @@ _dialog_save_cb(void *data, void *data2)
 
    /* Reusing this var */
    read_length = 0;
-   if (cfdata->error)
-     read_length = cfdata->error->size;
+   if (cfdata->error) read_length = cfdata->error->size;
 
    if (read_length)
      {
