@@ -6,14 +6,14 @@
 /* local subsystem functions */
 static void _e_manager_free(E_Manager *man);
 
-static int _e_manager_cb_window_show_request(void *data, int ev_type, void *ev);
-static int _e_manager_cb_window_configure(void *data, int ev_type, void *ev);
-static int _e_manager_cb_key_up(void *data, int ev_type, void *ev);
-static int _e_manager_cb_key_down(void *data, int ev_type, void *ev);
-static int _e_manager_cb_frame_extents_request(void *data, int ev_type, void *ev);
-static int _e_manager_cb_ping(void *data, int ev_type, void *ev);
-static int _e_manager_cb_screensaver_notify(void *data, int ev_type, void *ev);
-static int _e_manager_cb_client_message(void *data, int ev_type, void *ev);
+static Eina_Bool _e_manager_cb_window_show_request(void *data, int ev_type, void *ev);
+static Eina_Bool _e_manager_cb_window_configure(void *data, int ev_type, void *ev);
+static Eina_Bool _e_manager_cb_key_up(void *data, int ev_type, void *ev);
+static Eina_Bool _e_manager_cb_key_down(void *data, int ev_type, void *ev);
+static Eina_Bool _e_manager_cb_frame_extents_request(void *data, int ev_type, void *ev);
+static Eina_Bool _e_manager_cb_ping(void *data, int ev_type, void *ev);
+static Eina_Bool _e_manager_cb_screensaver_notify(void *data, int ev_type, void *ev);
+static Eina_Bool _e_manager_cb_client_message(void *data, int ev_type, void *ev);
 
 static Eina_Bool _e_manager_frame_extents_free_cb(const Eina_Hash *hash __UNUSED__,
 						  const void *key __UNUSED__,
@@ -59,10 +59,7 @@ e_manager_init(void)
 EAPI int
 e_manager_shutdown(void)
 {
-   E_Manager *man;
-
-   EINA_LIST_FREE(managers, man)
-     e_object_del(E_OBJECT(man));
+   E_FREE_LIST(managers, e_object_del);
 
    if (frame_extents)
      {
@@ -101,12 +98,9 @@ e_manager_new(Ecore_X_Window root, int num)
    ecore_x_window_size_get(man->root, &(man->w), &(man->h));
    if (e_config->use_virtual_roots)
      {
-	Ecore_X_Window mwin;
-	
 	man->win = ecore_x_window_override_new(man->root, man->x, man->y, man->w, man->h);
 	ecore_x_icccm_title_set(man->win, "Enlightenment Manager");
 	ecore_x_netwm_name_set(man->win, "Enlightenment Manager");
-	mwin = e_menu_grab_window_get();
 	ecore_x_window_raise(man->win);
      }
    else
@@ -164,7 +158,6 @@ e_manager_manage_windows(E_Manager *man)
 
    /* a manager is designated for each root. lets get all the windows in 
       the managers root */
-   printf("--------BD DO!!!\n");
    windows = ecore_x_window_children_get(man->root, &wnum);
    if (windows)
      {
@@ -176,12 +169,10 @@ e_manager_manage_windows(E_Manager *man)
 	       "KWM_DOCKWINDOW"
 	  };
 	Ecore_X_Atom atoms[3];
-        Ecore_X_Atom atom_xmbed, atom_kde_netwm_systray, atom_kwm_dockwindow,
-	  atom_window;
+        Ecore_X_Atom atom_xmbed, atom_kde_netwm_systray, atom_kwm_dockwindow;
 	unsigned char *data = NULL;
 	int count;
 	
-	atom_window = ECORE_X_ATOM_WINDOW;
 	ecore_x_atoms_get(atom_names, 3, atoms);
 	atom_xmbed = atoms[0];
 	atom_kde_netwm_systray = atoms[1];
@@ -260,6 +251,8 @@ e_manager_manage_windows(E_Manager *man)
 		  E_Desk       *desk = NULL;
 		  E_Border     *bd = NULL;
 		  unsigned int  id;
+		  char *path;
+		  Efreet_Desktop *desktop = NULL;
 
 		  /* get all information from window before it is 
 		   * reset by e_border_new */
@@ -286,6 +279,14 @@ e_manager_manage_windows(E_Manager *man)
 					    deskxy[0],
 					    deskxy[1]);
 
+		  path = ecore_x_window_prop_string_get(windows[i],
+							E_ATOM_DESKTOP_FILE);
+		  if (path)
+		    {
+		       desktop = efreet_desktop_get(path);
+		       free(path);
+		    }
+
 		    {
 		       bd = e_border_new(con, windows[i], 1, 0);
 		       if (bd)
@@ -296,6 +297,7 @@ e_manager_manage_windows(E_Manager *man)
 			     * be set according to the desk */
 //			    if (zone) e_border_zone_set(bd, zone);
 			    if (desk) e_border_desk_set(bd, desk);
+			    bd->desktop = desktop;
 			 }
 		    }
 	       }
@@ -324,15 +326,13 @@ EAPI void
 e_manager_show(E_Manager *man)
 {
    Eina_List *l;
+   E_Container *con;
    
    E_OBJECT_CHECK(man);
    E_OBJECT_TYPE_CHECK(man, E_MANAGER_TYPE);
    if (man->visible) return;
-   for (l = man->containers; l; l = l->next)
+   EINA_LIST_FOREACH(man->containers, l, con)
      {
-	E_Container *con;
-	
-	con = l->data;
 	e_container_show(con);
      }
    if (man->root != man->win)
@@ -358,15 +358,13 @@ EAPI void
 e_manager_hide(E_Manager *man)
 {
    Eina_List *l;
+   E_Container *con;
    
    E_OBJECT_CHECK(man);
    E_OBJECT_TYPE_CHECK(man, E_MANAGER_TYPE);
    if (!man->visible) return;
-   for (l = man->containers; l; l = l->next)
+   EINA_LIST_FOREACH(man->containers, l, con)
      {
-	E_Container *con;
-	
-	con = l->data;
 	e_container_hide(con);
      }
    if (man->root != man->win)
@@ -392,6 +390,7 @@ EAPI void
 e_manager_resize(E_Manager *man, int w, int h)
 {
    Eina_List *l;
+   E_Container *con;
    
    E_OBJECT_CHECK(man);
    E_OBJECT_TYPE_CHECK(man, E_MANAGER_TYPE);
@@ -401,11 +400,8 @@ e_manager_resize(E_Manager *man, int w, int h)
    if (man->root != man->win)
      ecore_x_window_resize(man->win, man->w, man->h);
 	
-   for (l = man->containers; l; l = l->next)
+   EINA_LIST_FOREACH(man->containers, l, con)
      {
-	E_Container *con;
-	
-	con = l->data;
 	e_container_resize(con, man->w, man->h);
      }
 
@@ -416,6 +412,7 @@ EAPI void
 e_manager_move_resize(E_Manager *man, int x, int y, int w, int h)
 {
    Eina_List *l;
+   E_Container *con;
    
    E_OBJECT_CHECK(man);
    E_OBJECT_TYPE_CHECK(man, E_MANAGER_TYPE);
@@ -429,11 +426,8 @@ e_manager_move_resize(E_Manager *man, int x, int y, int w, int h)
    man->h = h;
    ecore_x_window_move_resize(man->win, man->x, man->y, man->w, man->h);
 
-   for (l = man->containers; l; l = l->next)
+   EINA_LIST_FOREACH(man->containers, l, con)
      {
-	E_Container *con;
-	
-	con = l->data;
 	e_container_resize(con, man->w, man->h);
      }
 }
@@ -477,16 +471,15 @@ e_manager_current_get(void)
    int x, y;
    
    if (!managers) return NULL;
-   for (l = managers; l; l = l->next)
+   EINA_LIST_FOREACH(managers, l, man)
      {
-	man = l->data;
 	ecore_x_pointer_xy_get(man->win, &x, &y);
 	if (x == -1 && y == -1)
 	  continue;
 	if (E_INSIDE(x, y, man->x, man->y, man->w, man->h))
 	  return man;
      }
-   return managers->data;
+   return eina_list_data_get(managers);
 }
 
 EAPI E_Manager *
@@ -496,9 +489,8 @@ e_manager_number_get(int num)
    E_Manager *man;
    
    if (!managers) return NULL;
-   for (l = managers; l; l = l->next)
+   EINA_LIST_FOREACH(managers, l, man)
      {
-	man = l->data;
 	if (man->num == num)
 	  return man;
      }
@@ -509,12 +501,10 @@ EAPI void
 e_managers_keys_grab(void)
 {
    Eina_List *l;
+   E_Manager *man;
 
-   for (l = managers; l; l = l->next)
+   EINA_LIST_FOREACH(managers, l, man)
      {
-	E_Manager *man;
-	
-	man = l->data;
 	e_bindings_key_grab(E_BINDING_CONTEXT_ANY, man->root);
      }
 }
@@ -523,12 +513,10 @@ EAPI void
 e_managers_keys_ungrab(void)
 {
    Eina_List *l;
+   E_Manager *man;
    
-   for (l = managers; l; l = l->next)
+   EINA_LIST_FOREACH(managers, l, man)
      {
-	E_Manager *man;
-	
-	man = l->data;
 	e_bindings_key_ungrab(E_BINDING_CONTEXT_ANY, man->root);
      }
 }
@@ -539,21 +527,10 @@ _e_manager_free(E_Manager *man)
 {
    Eina_List *l;
 
-   while (man->handlers)
-     {
-	Ecore_Event_Handler *h;
-   
-	h = man->handlers->data;
-	man->handlers = eina_list_remove_list(man->handlers, man->handlers);
-	ecore_event_handler_del(h);
-     }
+   E_FREE_LIST(man->handlers, ecore_event_handler_del);
    l = man->containers;
    man->containers = NULL;
-   while (l)
-     {
-	e_object_del(E_OBJECT(l->data));
-	l = eina_list_remove_list(l, l);
-     }
+   E_FREE_LIST(l, e_object_del);
    if (man->root != man->win)
      {
 	ecore_x_window_free(man->win);
@@ -563,22 +540,22 @@ _e_manager_free(E_Manager *man)
    free(man);
 }
 
-static int
+static Eina_Bool
 _e_manager_cb_window_show_request(void *data, int ev_type __UNUSED__, void *ev)
 {
    E_Manager *man;
    Ecore_X_Event_Window_Show_Request *e;
-   
+
    man = data;
    e = ev;
    if (e_stolen_win_get(e->win)) return 1;
    if (ecore_x_window_parent_get(e->win) != man->root)
-     return 1;  /* try other handlers for this */
-   
+     return ECORE_CALLBACK_PASS_ON;  /* try other handlers for this */
+
      {
 	E_Container *con;
 	E_Border *bd;
-	
+
 	con = e_container_current_get(man);
 	if (!e_border_find_by_client_window(e->win))
 	  {
@@ -587,10 +564,10 @@ _e_manager_cb_window_show_request(void *data, int ev_type __UNUSED__, void *ev)
 	       ecore_x_window_show(e->win);
 	  }
      }
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
+static Eina_Bool
 _e_manager_cb_window_configure(void *data, int ev_type __UNUSED__, void *ev)
 {
    E_Manager *man;
@@ -598,12 +575,12 @@ _e_manager_cb_window_configure(void *data, int ev_type __UNUSED__, void *ev)
    
    man = data;
    e = ev;
-   if (e->win != man->root) return 1;
+   if (e->win != man->root) return ECORE_CALLBACK_PASS_ON;
    e_manager_resize(man, e->w, e->h);
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
+static Eina_Bool
 _e_manager_cb_key_down(void *data, int ev_type __UNUSED__, void *ev)
 {
    E_Manager *man;
@@ -612,14 +589,14 @@ _e_manager_cb_key_down(void *data, int ev_type __UNUSED__, void *ev)
    man = data;
    e = ev;
 
-   if (e->event_window != man->root) return 1;
+   if (e->event_window != man->root) return ECORE_CALLBACK_PASS_ON;
    if (e->root_window != man->root) man = _e_manager_get_for_root(e->root_window);
    if (e_bindings_key_down_event_handle(E_BINDING_CONTEXT_MANAGER, E_OBJECT(man), ev))
-     return 0;
-   return 1;
+     return ECORE_CALLBACK_DONE;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
+static Eina_Bool
 _e_manager_cb_key_up(void *data, int ev_type __UNUSED__, void *ev)
 {
    E_Manager *man;
@@ -628,14 +605,14 @@ _e_manager_cb_key_up(void *data, int ev_type __UNUSED__, void *ev)
    man = data;
    e = ev;
 
-   if (e->event_window != man->root) return 1;
+   if (e->event_window != man->root) return ECORE_CALLBACK_PASS_ON;
    if (e->root_window != man->root) man = _e_manager_get_for_root(e->root_window);
    if (e_bindings_key_up_event_handle(E_BINDING_CONTEXT_MANAGER, E_OBJECT(man), ev))
-     return 0;
-   return 1;
+     return ECORE_CALLBACK_DONE;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
+static Eina_Bool
 _e_manager_cb_frame_extents_request(void *data, int ev_type __UNUSED__, void *ev)
 {
    E_Manager *man;
@@ -653,7 +630,7 @@ _e_manager_cb_frame_extents_request(void *data, int ev_type __UNUSED__, void *ev
    con = e_container_current_get(man);
    e = ev;
 
-   if (ecore_x_window_parent_get(e->win) != man->root) return 1;
+   if (ecore_x_window_parent_get(e->win) != man->root) return ECORE_CALLBACK_PASS_ON;
 
    /* TODO:
     * * We need to check if we remember this window, and border locking is set
@@ -684,7 +661,6 @@ _e_manager_cb_frame_extents_request(void *data, int ev_type __UNUSED__, void *ev
    if (state)
      {
 	int maximized = 0;
-	int fullscreen = 0;
 
 	for (i = 0; i < num; i++)
 	  {
@@ -697,7 +673,6 @@ _e_manager_cb_frame_extents_request(void *data, int ev_type __UNUSED__, void *ev
 		  maximized++;
 		  break;
 		case ECORE_X_WINDOW_STATE_FULLSCREEN:
-		  fullscreen = 1;
 		  border = "borderless";
 		  key = border;
 		  break;
@@ -770,10 +745,10 @@ _e_manager_cb_frame_extents_request(void *data, int ev_type __UNUSED__, void *ev
    if (extents)
      ecore_x_netwm_frame_size_set(e->win, extents->l, extents->r, extents->t, extents->b);
 
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
+static Eina_Bool
 _e_manager_cb_ping(void *data, int ev_type __UNUSED__, void *ev)
 {
    E_Manager *man;
@@ -783,24 +758,24 @@ _e_manager_cb_ping(void *data, int ev_type __UNUSED__, void *ev)
    man = data;
    e = ev;
 
-   if (e->win != man->root) return 1;
+   if (e->win != man->root) return ECORE_CALLBACK_PASS_ON;
 
    bd = e_border_find_by_client_window(e->event_win);
-   if (!bd) return 1;
+   if (!bd) return ECORE_CALLBACK_PASS_ON;
 
    bd->ping_ok = 1;
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
+static Eina_Bool
 _e_manager_cb_timer_post_screensaver_lock(void *data __UNUSED__)
 {
    e_desklock_show_autolocked();
    timer_post_screensaver_lock = NULL;
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
-static int
+static Eina_Bool
 _e_manager_cb_screensaver_notify(void *data __UNUSED__, int ev_type __UNUSED__, void *ev)
 {
    Ecore_X_Event_Screensaver_Notify *e = ev;
@@ -827,17 +802,15 @@ _e_manager_cb_screensaver_notify(void *data __UNUSED__, int ev_type __UNUSED__,
 	       }
 	  }
      }
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
-_e_manager_cb_client_message(void *data, int ev_type, void *ev)
+static Eina_Bool
+_e_manager_cb_client_message(__UNUSED__ void *data, __UNUSED__ int ev_type, void *ev)
 {
-   E_Manager *man;
    Ecore_X_Event_Client_Message *e;
    E_Border *bd;
    
-   man = data;
    e = ev;
    
    if (e->message_type == ECORE_X_ATOM_NET_ACTIVE_WINDOW)
@@ -886,7 +859,7 @@ _e_manager_cb_client_message(void *data, int ev_type, void *ev)
 	  }
      }
 
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
 static Eina_Bool
@@ -894,7 +867,7 @@ _e_manager_frame_extents_free_cb(const Eina_Hash *hash __UNUSED__, const void *k
 				 void *data, void *fdata __UNUSED__)
 {
    free(data);
-   return 1;
+   return EINA_TRUE;
 }
 
 static E_Manager *
@@ -904,13 +877,12 @@ _e_manager_get_for_root(Ecore_X_Window root)
    E_Manager *man;
 
    if (!managers) return NULL;
-   for (l = managers; l; l = l->next)
+   EINA_LIST_FOREACH(managers, l, man)
      {
-	man = l->data;
 	if (man->root == root)
 	  return man;
      }
-   return managers->data;
+   return eina_list_data_get(managers);
 }
 
 
