@@ -33,84 +33,84 @@ struct _E_Menu_Category
 };
 
 /* local subsystem functions */
-static void _e_menu_free                          (E_Menu *m);
-static void _e_menu_item_free                     (E_Menu_Item *mi);
-static void _e_menu_item_realize                  (E_Menu_Item *mi);
-static void _e_menu_realize                       (E_Menu *m);
-static void _e_menu_items_layout_update           (E_Menu *m);
-static void _e_menu_item_unrealize                (E_Menu_Item *mi);
-static void _e_menu_unrealize                     (E_Menu *m);
-static void _e_menu_activate_internal             (E_Menu *m, E_Zone *zone);
-static void _e_menu_deactivate_all                (void);
-static void _e_menu_deactivate_above              (E_Menu *m);
-static void _e_menu_submenu_activate              (E_Menu_Item *mi);
-static void _e_menu_submenu_deactivate            (E_Menu_Item *mi);
-static void _e_menu_reposition                    (E_Menu *m);
-static int  _e_menu_active_call                   (void);
-static int _e_menu_realize_call                   (E_Menu_Item *mi);
-static void _e_menu_item_activate_next            (void);
-static void _e_menu_item_activate_previous        (void);
-static void _e_menu_item_activate_first           (void);
-static void _e_menu_item_activate_last            (void);
-static void _e_menu_item_activate_nth             (int n);
-static void _e_menu_item_activate_char            (const char * key_compose);
-static void _e_menu_activate_next                 (void);
-static void _e_menu_activate_previous             (void);
-static void _e_menu_activate_first                (void);
-static void _e_menu_activate_last                 (void);
+static void _e_menu_free(E_Menu *m);
+static void _e_menu_item_free(E_Menu_Item *mi);
+static void _e_menu_item_realize(E_Menu_Item *mi);
+static void _e_menu_realize(E_Menu *m);
+static void _e_menu_items_layout_update(E_Menu *m);
+static void _e_menu_item_unrealize(E_Menu_Item *mi);
+static void _e_menu_unrealize(E_Menu *m);
+static void _e_menu_activate_internal(E_Menu *m, E_Zone *zone);
+static void _e_menu_deactivate_all(void);
+static void _e_menu_deactivate_above(E_Menu *m);
+static void _e_menu_submenu_activate(E_Menu_Item *mi);
+static void _e_menu_submenu_deactivate(E_Menu_Item *mi);
+static void _e_menu_reposition(E_Menu *m);
+static int _e_menu_active_call(void);
+static int _e_menu_realize_call(E_Menu_Item *mi);
+static void _e_menu_item_activate_next(void);
+static void _e_menu_item_activate_previous(void);
+static void _e_menu_item_activate_first(void);
+static void _e_menu_item_activate_last(void);
+static void _e_menu_item_activate_nth(int n);
+static void _e_menu_item_activate_char(const char * key_compose);
+static void _e_menu_activate_next(void);
+static void _e_menu_activate_previous(void);
+static void _e_menu_activate_first(void);
+static void _e_menu_activate_last(void);
 #if 0
-static void _e_menu_activate_nth                  (int n);
+static void _e_menu_activate_nth(int n);
 #endif
-static E_Menu *_e_menu_active_get                 (void);
-static E_Menu_Item *_e_menu_item_active_get       (void);
-static Eina_List * _e_menu_list_item_active_get   (void);
-static int  _e_menu_outside_bounds_get            (int xdir, int ydir);
-static void _e_menu_scroll_by                     (int dx, int dy);
-static void _e_menu_mouse_autoscroll_check        (void);
-static void _e_menu_item_ensure_onscreen          (E_Menu_Item *mi);
-static int  _e_menu_auto_place                    (E_Menu *m, int x, int y, int w, int h);
-static void _e_menu_cb_intercept_item_move        (void *data, Evas_Object *o, Evas_Coord x, Evas_Coord y);
-static void _e_menu_cb_intercept_item_resize      (void *data, Evas_Object *o, Evas_Coord w, Evas_Coord h);
-static void _e_menu_cb_intercept_container_move   (void *data, Evas_Object *o, Evas_Coord x, Evas_Coord y);
-static void _e_menu_cb_intercept_container_resize (void *data, Evas_Object *o, Evas_Coord w, Evas_Coord h);
-static void _e_menu_cb_ecore_evas_resize          (Ecore_Evas *ee);
-static void _e_menu_cb_item_in                    (void *data, Evas *evas, Evas_Object *obj, void *event_info);
-static void _e_menu_cb_item_out                   (void *data, Evas *evas, Evas_Object *obj, void *event_info);
-static int  _e_menu_cb_key_down                   (void *data, int type, void *event);
-static int  _e_menu_cb_key_up                     (void *data, int type, void *event);
-static int  _e_menu_cb_mouse_down                 (void *data, int type, void *event);
-static int  _e_menu_cb_mouse_up                   (void *data, int type, void *event);
-static int  _e_menu_cb_mouse_move                 (void *data, int type, void *event);
-static int  _e_menu_cb_mouse_wheel                (void *data, int type, void *event);
-static int  _e_menu_cb_scroll_animator            (void *data);
-static int  _e_menu_cb_window_shape               (void *data, int ev_type, void *ev);
-
-static void      _e_menu_cb_item_submenu_post_default  (void *data, E_Menu *m, E_Menu_Item *mi);
-static Evas_Bool _e_menu_categories_free_cb            (const Eina_Hash *hash, const void *key, void *data, void *fdata);
+static E_Menu *_e_menu_active_get(void);
+static E_Menu_Item *_e_menu_item_active_get(void);
+static Eina_List *_e_menu_list_item_active_get(void);
+static int _e_menu_outside_bounds_get(int xdir, int ydir);
+static void _e_menu_scroll_by(int dx, int dy);
+static void _e_menu_mouse_autoscroll_check(void);
+static void _e_menu_item_ensure_onscreen(E_Menu_Item *mi);
+static int _e_menu_auto_place(E_Menu *m, int x, int y, int w, int h);
+static void _e_menu_cb_intercept_item_move(void *data, Evas_Object *o, Evas_Coord x, Evas_Coord y);
+static void _e_menu_cb_intercept_item_resize(void *data, Evas_Object *o, Evas_Coord w, Evas_Coord h);
+static void _e_menu_cb_intercept_container_move(void *data, Evas_Object *o, Evas_Coord x, Evas_Coord y);
+static void _e_menu_cb_intercept_container_resize(void *data, Evas_Object *o, Evas_Coord w, Evas_Coord h);
+static void _e_menu_cb_ecore_evas_resize(Ecore_Evas *ee);
+static void _e_menu_cb_item_in(void *data, Evas *evas, Evas_Object *obj, void *event_info);
+static void _e_menu_cb_item_out(void *data, Evas *evas, Evas_Object *obj, void *event_info);
+static Eina_Bool _e_menu_cb_key_down(void *data, int type, void *event);
+static Eina_Bool _e_menu_cb_key_up(void *data, int type, void *event);
+static Eina_Bool _e_menu_cb_mouse_down(void *data, int type, void *event);
+static Eina_Bool _e_menu_cb_mouse_up(void *data, int type, void *event);
+static Eina_Bool _e_menu_cb_mouse_move(void *data, int type, void *event);
+static Eina_Bool _e_menu_cb_mouse_wheel(void *data, int type, void *event);
+static Eina_Bool _e_menu_cb_scroll_animator(void *data);
+static Eina_Bool _e_menu_cb_window_shape(void *data, int ev_type, void *ev);
+static void _e_menu_cb_item_submenu_post_default(void *data, E_Menu *m, E_Menu_Item *mi);
+static Eina_Bool _e_menu_categories_free_cb(const Eina_Hash *hash, const void *key, void *data, void *fdata);
 
 /* local subsystem globals */
-static Ecore_X_Window       _e_menu_win                 = 0;
-static Eina_List           *_e_active_menus             = NULL;
-static E_Menu_Item         *_e_active_menu_item         = NULL;
+static Ecore_X_Window _e_menu_win = 0;
+static Eina_List *_e_active_menus = NULL;
+static Eina_Hash *_e_menu_hash = NULL;
+static E_Menu_Item *_e_active_menu_item = NULL;
 /*static Eina_Hash	   *_e_menu_category_items	= NULL;*/
-static Eina_Hash	   *_e_menu_categories		= NULL;
-static Ecore_X_Time         _e_menu_activate_time       = 0;
-static int                  _e_menu_activate_floating   = 0;
-static int                  _e_menu_activate_maybe_drag = 0;
-static int                  _e_menu_activate_dragging   = 0;
-static Ecore_Animator      *_e_menu_scroll_animator     = NULL;
-static double               _e_menu_scroll_start        = 0.0;
-static int                  _e_menu_x                   = 0;
-static int                  _e_menu_y                   = 0;
-static Ecore_X_Time         _e_menu_time                = 0;
-static int                  _e_menu_autoscroll_x        = 0;
-static int                  _e_menu_autoscroll_y        = 0;
-static Ecore_Event_Handler *_e_menu_key_down_handler     = NULL;
-static Ecore_Event_Handler *_e_menu_key_up_handler       = NULL;
-static Ecore_Event_Handler *_e_menu_mouse_down_handler   = NULL;
-static Ecore_Event_Handler *_e_menu_mouse_up_handler     = NULL;
-static Ecore_Event_Handler *_e_menu_mouse_move_handler   = NULL;
-static Ecore_Event_Handler *_e_menu_mouse_wheel_handler  = NULL;
+static Eina_Hash *_e_menu_categories = NULL;
+static Ecore_X_Time _e_menu_activate_time = 0;
+static int _e_menu_activate_floating = 0;
+static int _e_menu_activate_maybe_drag = 0;
+static int _e_menu_activate_dragging = 0;
+static Ecore_Animator *_e_menu_scroll_animator = NULL;
+static double _e_menu_scroll_start = 0.0;
+static int _e_menu_x = 0;
+static int _e_menu_y = 0;
+static Ecore_X_Time _e_menu_time = 0;
+static int _e_menu_autoscroll_x = 0;
+static int _e_menu_autoscroll_y = 0;
+static Ecore_Event_Handler *_e_menu_key_down_handler = NULL;
+static Ecore_Event_Handler *_e_menu_key_up_handler = NULL;
+static Ecore_Event_Handler *_e_menu_mouse_down_handler = NULL;
+static Ecore_Event_Handler *_e_menu_mouse_up_handler = NULL;
+static Ecore_Event_Handler *_e_menu_mouse_move_handler = NULL;
+static Ecore_Event_Handler *_e_menu_mouse_wheel_handler = NULL;
 static Ecore_Event_Handler *_e_menu_window_shape_handler = NULL;
 
 static Eina_List *
@@ -141,20 +141,36 @@ _e_menu_list_free_unref(Eina_List *l)
 EAPI int
 e_menu_init(void)
 {
-   _e_menu_key_down_handler     = ecore_event_handler_add(ECORE_EVENT_KEY_DOWN,           _e_menu_cb_key_down,    NULL);
-   _e_menu_key_up_handler       = ecore_event_handler_add(ECORE_EVENT_KEY_UP,             _e_menu_cb_key_up,      NULL);
-   _e_menu_mouse_down_handler   = ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_DOWN,  _e_menu_cb_mouse_down,  NULL);
-   _e_menu_mouse_up_handler     = ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_UP,    _e_menu_cb_mouse_up,    NULL);
-   _e_menu_mouse_move_handler   = ecore_event_handler_add(ECORE_EVENT_MOUSE_MOVE,         _e_menu_cb_mouse_move,  NULL);
-   _e_menu_mouse_wheel_handler  = ecore_event_handler_add(ECORE_EVENT_MOUSE_WHEEL,        _e_menu_cb_mouse_wheel, NULL);
-   _e_menu_window_shape_handler = ecore_event_handler_add(ECORE_X_EVENT_WINDOW_SHAPE,     _e_menu_cb_window_shape, NULL);
+   _e_menu_key_down_handler = 
+     ecore_event_handler_add(ECORE_EVENT_KEY_DOWN, _e_menu_cb_key_down, NULL);
+   _e_menu_key_up_handler = 
+     ecore_event_handler_add(ECORE_EVENT_KEY_UP, _e_menu_cb_key_up, NULL);
+   _e_menu_mouse_down_handler = 
+     ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_DOWN, 
+                             _e_menu_cb_mouse_down, NULL);
+   _e_menu_mouse_up_handler = 
+     ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_UP, 
+                             _e_menu_cb_mouse_up, NULL);
+   _e_menu_mouse_move_handler = 
+     ecore_event_handler_add(ECORE_EVENT_MOUSE_MOVE, 
+                             _e_menu_cb_mouse_move, NULL);
+   _e_menu_mouse_wheel_handler = 
+     ecore_event_handler_add(ECORE_EVENT_MOUSE_WHEEL, 
+                             _e_menu_cb_mouse_wheel, NULL);
+   _e_menu_window_shape_handler = 
+     ecore_event_handler_add(ECORE_X_EVENT_WINDOW_SHAPE, 
+                             _e_menu_cb_window_shape, NULL);
    _e_menu_categories = eina_hash_string_superfast_new(NULL);
+
+   if (!_e_menu_hash) _e_menu_hash = eina_hash_string_superfast_new(NULL);
    return 1;
 }
 
 EAPI int
 e_menu_shutdown(void)
 {
+   E_Menu *m;
+
    E_FN_DEL(ecore_event_handler_del, _e_menu_key_down_handler);
    E_FN_DEL(ecore_event_handler_del, _e_menu_key_up_handler);
    E_FN_DEL(ecore_event_handler_del, _e_menu_mouse_down_handler);
@@ -163,16 +179,10 @@ e_menu_shutdown(void)
    E_FN_DEL(ecore_event_handler_del, _e_menu_mouse_wheel_handler);
    E_FN_DEL(ecore_event_handler_del, _e_menu_window_shape_handler);
 
-   while (_e_active_menus)
+   EINA_LIST_FREE(_e_active_menus, m)
      {
-	E_Menu *m;
-
-	m = _e_active_menus->data;
 	m->active = 0;
 	_e_menu_unrealize(m);
-	_e_active_menus = 
-          eina_list_remove_list(_e_active_menus, _e_active_menus);
-
 	m->in_active_list = 0;
 	e_object_unref(E_OBJECT(m));
      }
@@ -184,6 +194,12 @@ e_menu_shutdown(void)
 	_e_menu_categories = NULL;
      }
 
+   if (_e_menu_hash)
+     {
+        eina_hash_free(_e_menu_hash);
+        _e_menu_hash = NULL;
+     }
+
    return 1;
 }
 
@@ -295,6 +311,7 @@ e_menu_activate_mouse(E_Menu *m, E_Zone *zone, int x, int y, int w, int h, int d
 	  m->cur.y = y + h - m->cur.h;
 	break;
       case E_MENU_POP_DIRECTION_RIGHT:
+	_e_menu_realize(m);
 	m->cur.x = x + w;
 	m->cur.y = y;
 	break;
@@ -762,7 +779,7 @@ e_menu_item_toggle_get(E_Menu_Item *mi)
 }
 
 EAPI void
-e_menu_item_callback_set(E_Menu_Item *mi,  void (*func) (void *data, E_Menu *m, E_Menu_Item *mi), void *data)
+e_menu_item_callback_set(E_Menu_Item *mi, void (*func) (void *data, E_Menu *m, E_Menu_Item *mi), void *data)
 {
    E_OBJECT_CHECK(mi);
    E_OBJECT_TYPE_CHECK(mi, E_MENU_ITEM_TYPE);
@@ -771,7 +788,7 @@ e_menu_item_callback_set(E_Menu_Item *mi,  void (*func) (void *data, E_Menu *m,
 }
 
 EAPI void
-e_menu_item_realize_callback_set(E_Menu_Item *mi,  void (*func) (void *data, E_Menu *m, E_Menu_Item *mi), void *data)
+e_menu_item_realize_callback_set(E_Menu_Item *mi, void (*func) (void *data, E_Menu *m, E_Menu_Item *mi), void *data)
 {
    E_OBJECT_CHECK(mi);
    E_OBJECT_TYPE_CHECK(mi, E_MENU_ITEM_TYPE);
@@ -780,7 +797,7 @@ e_menu_item_realize_callback_set(E_Menu_Item *mi,  void (*func) (void *data, E_M
 }
 
 EAPI void
-e_menu_item_submenu_pre_callback_set(E_Menu_Item *mi,  void (*func) (void *data, E_Menu *m, E_Menu_Item *mi), void *data)
+e_menu_item_submenu_pre_callback_set(E_Menu_Item *mi, void (*func) (void *data, E_Menu *m, E_Menu_Item *mi), void *data)
 {
    E_OBJECT_CHECK(mi);
    E_OBJECT_TYPE_CHECK(mi, E_MENU_ITEM_TYPE);
@@ -791,7 +808,7 @@ e_menu_item_submenu_pre_callback_set(E_Menu_Item *mi,  void (*func) (void *data,
 }
 
 EAPI void
-e_menu_item_submenu_post_callback_set(E_Menu_Item *mi,  void (*func) (void *data, E_Menu *m, E_Menu_Item *mi), void *data)
+e_menu_item_submenu_post_callback_set(E_Menu_Item *mi, void (*func) (void *data, E_Menu *m, E_Menu_Item *mi), void *data)
 {
    E_OBJECT_CHECK(mi);
    E_OBJECT_TYPE_CHECK(mi, E_MENU_ITEM_TYPE);
@@ -800,7 +817,7 @@ e_menu_item_submenu_post_callback_set(E_Menu_Item *mi,  void (*func) (void *data
 }
 
 EAPI void
-e_menu_item_drag_callback_set(E_Menu_Item *mi,  void (*func) (void *data, E_Menu *m, E_Menu_Item *mi), void *data)
+e_menu_item_drag_callback_set(E_Menu_Item *mi, void (*func) (void *data, E_Menu *m, E_Menu_Item *mi), void *data)
 {
    E_OBJECT_CHECK(mi);
    E_OBJECT_TYPE_CHECK(mi, E_MENU_ITEM_TYPE);
@@ -835,7 +852,15 @@ e_menu_item_active_set(E_Menu_Item *mi, int active)
 	if (mi->toggle_object)
 	  edje_object_signal_emit(mi->toggle_object, "e,state,selected", "e");
 	if (mi->icon_key)
-	  edje_object_signal_emit(mi->icon_object, "e,state,selected", "e");
+	  {
+             if (mi->icon_object)
+               {
+                  if (strcmp(evas_object_type_get(mi->icon_object), "e_icon"))
+                    edje_object_signal_emit(mi->icon_object, "e,state,selected", "e");
+                  else
+                    e_icon_selected_set(mi->icon_object, EINA_TRUE);
+               }
+	  }
 	edje_object_signal_emit(mi->menu->bg_object, "e,state,selected", "e");
 	_e_menu_submenu_activate(mi);
      }
@@ -854,7 +879,15 @@ e_menu_item_active_set(E_Menu_Item *mi, int active)
 	if (mi->toggle_object)
 	  edje_object_signal_emit(mi->toggle_object, "e,state,unselected", "e");
 	if (mi->icon_key)
-	  edje_object_signal_emit(mi->icon_object, "e,state,unselected", "e");
+	  {
+             if (mi->icon_object)
+               {
+                  if (strcmp(evas_object_type_get(mi->icon_object), "e_icon"))
+                    edje_object_signal_emit(mi->icon_object, "e,state,unselected", "e");
+                  else
+                    e_icon_selected_set(mi->icon_object, EINA_FALSE);
+               }
+	  }
 	edje_object_signal_emit(mi->menu->bg_object, "e,state,unselected", "e");
 	_e_menu_submenu_deactivate(mi);
      }
@@ -993,6 +1026,7 @@ e_menu_idler_before(void)
 				 break;
 			      }
 			 }
+		       // TODO: This is meaningless
 		       changed = 0;
 		    }
 		  if (changed)
@@ -1017,7 +1051,7 @@ e_menu_idler_before(void)
 	  }
      }
    /* del refcount to all menus we worked with */
-   tmp = _e_menu_list_free_unref(tmp);
+   _e_menu_list_free_unref(tmp);
 
    if (!_e_active_menus)
      {
@@ -1036,6 +1070,18 @@ e_menu_grab_window_get(void)
   return _e_menu_win;
 }
 
+EAPI E_Menu *
+e_menu_find_by_window(Ecore_X_Window win)
+{
+   E_Menu *m;
+   
+   m = eina_hash_find(_e_menu_hash, e_util_winid_str_get(win));
+   if ((m) && (m->evas_win != win))
+     return NULL;
+   return m;
+}
+
+
 /* local subsystem functions */
 static void
 _e_menu_free(E_Menu *m)
@@ -1059,9 +1105,7 @@ _e_menu_free(E_Menu *m)
    E_FREE(m->shape_rects);
    m->shape_rects_num = 0;
    EINA_LIST_FOREACH_SAFE(m->items, l, l_next, mi)
-     {
-	e_object_del(E_OBJECT(mi));
-     }
+     e_object_del(E_OBJECT(mi));
    if (m->in_active_list)
      {
 	_e_active_menus = eina_list_remove(_e_active_menus, m);
@@ -1277,8 +1321,9 @@ _e_menu_item_realize(E_Menu_Item *mi)
 		    {
 		       o = e_icon_add(mi->menu->evas);
 		       mi->icon_object = o;
+                       e_icon_preload_set(mi->icon_object, 1);
 		       e_icon_file_set(o, mi->icon);
-		       e_icon_fill_inside_set(o, 1);
+		       e_icon_fill_inside_set(mi->icon_object, 1);
 		       e_icon_size_get(mi->icon_object, &icon_w, &icon_h);
 		    }
 	       }
@@ -1381,8 +1426,8 @@ _e_menu_item_realize(E_Menu_Item *mi)
 	evas_object_color_set(o, 0, 0, 0, 0);
 	evas_object_layer_set(o, 1);
 	evas_object_repeat_events_set(o, 1);
-	evas_object_event_callback_add(o, EVAS_CALLBACK_MOUSE_IN,  
-                                       _e_menu_cb_item_in,  mi);
+	evas_object_event_callback_add(o, EVAS_CALLBACK_MOUSE_IN,
+                                       _e_menu_cb_item_in, mi);
 	evas_object_event_callback_add(o, EVAS_CALLBACK_MOUSE_OUT, 
                                        _e_menu_cb_item_out, mi);
 	evas_object_show(o);
@@ -1411,6 +1456,7 @@ _e_menu_realize(E_Menu *m)
                   m->cur.x, m->cur.y, m->cur.w, m->cur.h, 1, 1,
                   &(m->evas_win));
    e_canvas_add(m->ecore_evas);
+   eina_hash_add(_e_menu_hash, e_util_winid_str_get(m->evas_win), m);
    m->shape = e_container_shape_add(m->zone->container);
    e_container_shape_move(m->shape, m->cur.x, m->cur.y);
    e_container_shape_resize(m->shape, m->cur.w, m->cur.h);
@@ -1420,11 +1466,12 @@ _e_menu_realize(E_Menu *m)
    evas_event_freeze(m->evas);
    /* move cursor out to avoid event cycles during setup */
    evas_event_feed_mouse_in(m->evas, ecore_x_current_time_get(), NULL);
-   evas_event_feed_mouse_move(m->evas, -1000000, -1000000, ecore_x_current_time_get(), NULL);
+   evas_event_feed_mouse_move(m->evas, -1000000, -1000000, 
+                              ecore_x_current_time_get(), NULL);
    ecore_x_window_shape_events_select(m->evas_win, 1);
    ecore_evas_name_class_set(m->ecore_evas, "E", "_e_menu_window");
    ecore_evas_title_set(m->ecore_evas, "E Menu");
-   
+
    o = edje_object_add(m->evas);
    m->bg_object = o;
    evas_object_name_set(o, "menu/background");
@@ -1436,7 +1483,7 @@ _e_menu_realize(E_Menu *m)
    if (ok)
      {
 	const char *shape_option;
-	
+
 	shape_option = edje_object_data_get(o, "shaped");
 	if (shape_option)
 	  {
@@ -1456,11 +1503,16 @@ _e_menu_realize(E_Menu *m)
 	if (e_config->use_composite)
 	  {
 	     ecore_evas_alpha_set(m->ecore_evas, m->shaped);
-	     m->evas_win = ecore_evas_software_x11_window_get(m->ecore_evas);
+
+             eina_hash_del(_e_menu_hash, e_util_winid_str_get(m->evas_win), m);
+             m->evas_win = ecore_evas_software_x11_window_get(m->ecore_evas);
+             eina_hash_add(_e_menu_hash, e_util_winid_str_get(m->evas_win), m);
 	  }
 	else
 	  ecore_evas_shaped_set(m->ecore_evas, m->shaped);
      }
+
+   ecore_x_netwm_window_type_set(m->evas_win, ECORE_X_WINDOW_TYPE_MENU);
    
    o = e_box_add(m->evas);
    m->container_object = o;
@@ -1472,9 +1524,7 @@ _e_menu_realize(E_Menu *m)
    edje_object_part_swallow(m->bg_object, "e.swallow.content", m->container_object);
    
    EINA_LIST_FOREACH(m->items, l, mi)
-     {
-	_e_menu_item_realize(mi);
-     }
+     _e_menu_item_realize(mi);
 
    _e_menu_items_layout_update(m);
    e_box_thaw(m->container_object);
@@ -1667,7 +1717,7 @@ _e_menu_items_layout_update(E_Menu *m)
 	     e_box_thaw(mi->container_object);
 	  }
      }
-   e_box_min_size_get(m->container_object, &bw, &bh);
+   e_box_size_min_get(m->container_object, &bw, &bh);
    edje_extern_object_min_size_set(m->container_object, bw, bh);
    edje_extern_object_max_size_set(m->container_object, bw, bh);
    edje_object_part_swallow(m->bg_object, "e.swallow.content", m->container_object);
@@ -1680,7 +1730,7 @@ _e_menu_items_layout_update(E_Menu *m)
 static void
 _e_menu_item_unrealize(E_Menu_Item *mi)
 {
-   e_box_freeze(mi->container_object);
+   if (mi->container_object) e_box_freeze(mi->container_object);
    if (mi->separator_object) evas_object_del(mi->separator_object);
    mi->separator_object = NULL;
    if (mi->bg_object) evas_object_del(mi->bg_object);
@@ -1714,9 +1764,7 @@ _e_menu_unrealize(E_Menu *m)
    m->shape = NULL;
    e_box_freeze(m->container_object);
    EINA_LIST_FOREACH(m->items, l, mi)
-     {
-	_e_menu_item_unrealize(mi);
-     }
+     _e_menu_item_unrealize(mi);
    if (m->header.icon) evas_object_del(m->header.icon);
    m->header.icon = NULL;
    if (m->bg_object) evas_object_del(m->bg_object);
@@ -1731,6 +1779,7 @@ _e_menu_unrealize(E_Menu *m)
    ecore_evas_free(m->ecore_evas);
    m->ecore_evas = NULL;
    m->evas = NULL;
+   eina_hash_del(_e_menu_hash, e_util_winid_str_get(m->evas_win), m);
    m->evas_win = 0;
 }
 
@@ -1781,14 +1830,12 @@ _e_menu_activate_internal(E_Menu *m, E_Zone *zone)
    /* the foreign menu items */
    if (m->category)
      {
-	cat = evas_hash_find(_e_menu_categories, m->category);
+	cat = eina_hash_find(_e_menu_categories, m->category);
 	if (cat)
 	  {
 	     E_Menu_Category_Callback *cb;
 	     EINA_LIST_FOREACH(cat->callbacks, l, cb)
-	       {
-		  if (cb->create) cb->create(m, cat->data, cb->data);
-	       }
+               if (cb->create) cb->create(m, cat->data, cb->data);
 	  }
      }
    m->cur.visible = 1;
@@ -1904,9 +1951,7 @@ _e_menu_reposition(E_Menu *m)
    tmp = _e_active_menus_copy_ref();
 
    EINA_LIST_FOREACH(m->items, l, mi)
-     {
-	if ((mi->active) && (mi->submenu)) _e_menu_reposition(mi->submenu);
-     }
+     if ((mi->active) && (mi->submenu)) _e_menu_reposition(mi->submenu);
 
    _e_menu_list_free_unref(tmp);
 }
@@ -1957,19 +2002,15 @@ _e_menu_item_activate_next(void)
      {
 	/* Look at the next item and then cycle until we're not on
 	 * a separator. */
-	if (!(ll->next)) 
-	  ll = mi->menu->items;
-	else
-	   ll = ll->next;
-	mi = ll->data;
-	while (mi->separator || mi->disable)
+	do
 	  { 
-	     if (!(ll->next)) 
+	     if (!eina_list_next(ll)) 
 	       ll = mi->menu->items;
 	     else
-	       ll = ll->next;
-	     mi = ll->data;
+	       ll = eina_list_next(ll);
+	     mi = eina_list_data_get(ll);
 	  }
+	while (mi->separator || mi->disable);
 
 	e_menu_item_active_set(mi, 1);
 	_e_menu_item_ensure_onscreen(mi);
@@ -1991,19 +2032,15 @@ _e_menu_item_activate_previous(void)
      {
 	/* Look at the prev item and then cycle until we're not on
 	 * a separator. */
-	if (!(ll->prev)) 
-	  ll = eina_list_last(ll);
-	else
-	   ll = ll->prev;
-	mi = ll->data;
-	while (mi->separator || mi->disable)
+	do
 	  { 
-	     if (!(ll->prev)) 
+	     if (!eina_list_prev(ll)) 
 	       ll = eina_list_last(ll);
 	     else
-	       ll = ll->prev;
-	     mi = ll->data;
+	       ll = eina_list_prev(ll);
+	     mi = eina_list_data_get(ll);
 	  }
+	while ((mi->separator) || (mi->disable));
 
 	e_menu_item_active_set(mi, 1);
 	_e_menu_item_ensure_onscreen(mi);
@@ -2024,11 +2061,11 @@ _e_menu_item_activate_first(void)
    if (m)
      {
 	ll = m->items;
-	mi = ll->data;
-	while ((mi->separator) && (ll->next))
+	mi = eina_list_data_get(ll);
+	while ((mi->separator) && eina_list_next(ll))
 	  {
-	     ll = ll->next;
-	     mi = ll->data;
+	     ll = eina_list_next(ll);
+	     mi = eina_list_data_get(ll);
 	  }
 	if (mi->separator) return;
 	e_menu_item_active_set(mi, 1);
@@ -2049,11 +2086,11 @@ _e_menu_item_activate_last(void)
    if (m)
      {
 	ll = eina_list_last(m->items);
-	mi = ll->data;
-	while ((mi->separator) && (ll->prev))
+	mi = eina_list_data_get(ll);
+	while ((mi->separator) && eina_list_prev(ll))
 	  {
-	     ll = ll->prev;
-	     mi = ll->data;
+	     ll = eina_list_prev(ll);
+	     mi = eina_list_data_get(ll);
 	  }
 	if (mi->separator) return;
 	e_menu_item_active_set(mi, 1);
@@ -2103,7 +2140,7 @@ _e_menu_item_activate_char(const char * key_compose)
    if (!m) 
      {
 	if (!_e_active_menus) return;
-	m = _e_active_menus->data;
+	m = eina_list_data_get(_e_active_menus);
 	if (!m) return;
      }
 
@@ -2112,7 +2149,7 @@ _e_menu_item_activate_char(const char * key_compose)
    if (!ll)
      {
 	ll = m->items;
-	mi = ll->data;
+	mi = eina_list_data_get(ll);
 	/* Only check the current item if it wasn't active before. */
 	if (!mi->separator && mi->label && !strncasecmp(key_compose, mi->label, strlen(key_compose))) 
 	  {
@@ -2124,32 +2161,27 @@ _e_menu_item_activate_char(const char * key_compose)
 
    ll_orig = ll;
 
-   mi = ll->data;
-   if (!(ll->next)) 
+   mi = eina_list_data_get(ll);
+   if (!eina_list_next(ll)) 
      ll = mi->menu->items;
    else
-     ll = ll->next;
-   mi = ll->data;
+     ll = eina_list_next(ll);
+   mi = eina_list_data_get(ll);
 
    /* While we don't have a label OR we don't match  AND we haven't
     * wrapped around */
    while ((!mi->label || strncasecmp(key_compose, mi->label, strlen(key_compose))) 
-	 && ll != ll_orig)
+          && ll != ll_orig)
      {
-	if (!(ll->next)) 
-	  ll = mi->menu->items;
-	else
-	  ll = ll->next;
-	mi = ll->data;
-
-	while (mi->separator)
-	  { 
-	     if (!(ll->next)) 
-	       ll = mi->menu->items;
-	     else
-	       ll = ll->next;
-	     mi = ll->data;
+	do
+	  {
+	     if (!eina_list_next(ll)) 
+     	       ll = mi->menu->items;
+     	     else
+     	       ll = eina_list_next(ll);
+     	     mi = eina_list_data_get(ll);
 	  }
+	while (mi->separator);
      }
 
    e_menu_item_active_set(mi, 1);
@@ -2169,7 +2201,7 @@ _e_menu_activate_next(void)
 	  {
 	     if (mi->submenu->items)
 	       {
-		  mi = mi->submenu->items->data;
+		  mi = eina_list_data_get(mi->submenu->items);
 		  e_menu_item_active_set(mi, 1);
 		  _e_menu_item_ensure_onscreen(mi);
 	       }
@@ -2206,14 +2238,14 @@ _e_menu_activate_first(void)
    Eina_List *ll;
    
    if (!_e_active_menus) return;
-   m = _e_active_menus->data;
+   m = eina_list_data_get(_e_active_menus);
    if (!m->items) return;
    ll = m->items;
-   mi = ll->data;
-   while ((mi->separator) && (ll->next))
+   mi = eina_list_data_get(ll);
+   while ((mi->separator) && eina_list_next(ll))
      {
-	ll = ll->next;
-	mi = ll->data;
+	ll = eina_list_next(ll);
+	mi = eina_list_data_get(ll);
      }
    if (mi->separator) return;
    e_menu_item_active_set(mi, 1);   
@@ -2228,14 +2260,14 @@ _e_menu_activate_last(void)
    Eina_List *ll;
    
    if (!_e_active_menus) return;
-   m = _e_active_menus->data;
+   m = eina_list_data_get(_e_active_menus);
    if (!m->items) return;
    ll = eina_list_last(m->items);
-   mi = ll->data;
-   while ((mi->separator) && (ll->prev))
+   mi = eina_list_data_get(ll);
+   while ((mi->separator) && eina_list_prev(ll))
      {
-	ll = ll->prev;
-	mi = ll->data;
+	ll = eina_list_prev(ll);
+	mi = eina_list_data_get(ll);
      }
    if (mi->separator) return;
    e_menu_item_active_set(mi, 1);   
@@ -2380,7 +2412,7 @@ _e_menu_mouse_autoscroll_check(void)
 	  {
 	     E_Menu *m;
 	     
-	     m = _e_active_menus->data;
+	     m = eina_list_data_get(_e_active_menus);
 	     if (_e_menu_x + e_config->menu_autoscroll_cursor_margin >= (m->zone->w - 1))
 	       {
 		  if (_e_menu_outside_bounds_get(1, 0)) autoscroll_x = 1;
@@ -2521,7 +2553,7 @@ _e_menu_cb_ecore_evas_resize(Ecore_Evas *ee)
 }
 
 static void
-_e_menu_cb_item_in(void *data, Evas *evas, Evas_Object *obj, void *event_info)
+_e_menu_cb_item_in(void *data, __UNUSED__ Evas *evas, __UNUSED__ Evas_Object *obj, __UNUSED__ void *event_info)
 {
    E_Menu_Item *mi;
    
@@ -2530,7 +2562,7 @@ _e_menu_cb_item_in(void *data, Evas *evas, Evas_Object *obj, void *event_info)
 }
 
 static void
-_e_menu_cb_item_out(void *data, Evas *evas, Evas_Object *obj, void *event_info)
+_e_menu_cb_item_out(void *data, __UNUSED__ Evas *evas, __UNUSED__ Evas_Object *obj, void *event_info)
 {
    E_Menu_Item *mi;
    Evas_Event_Mouse_In *ev;
@@ -2553,37 +2585,28 @@ _e_menu_cb_item_out(void *data, Evas *evas, Evas_Object *obj, void *event_info)
      }
 }
 
-static int
-_e_menu_cb_key_down(void *data, int type, void *event)
+static Eina_Bool
+_e_menu_cb_key_down(__UNUSED__ void *data, __UNUSED__ int type, void *event)
 {
    Ecore_Event_Key *ev;
    
    ev = event;
-   if (ev->window != _e_menu_win) return 1;
-   if      ((!strcmp(ev->key, "Up")) ||
-	    (!strcmp(ev->key, "KP_Up")))
+   if (ev->window != _e_menu_win) return ECORE_CALLBACK_PASS_ON;
+   if ((!strcmp(ev->key, "Up")) || (!strcmp(ev->key, "KP_Up")))
      _e_menu_item_activate_previous();
-   else if ((!strcmp(ev->key, "Down")) ||
-	    (!strcmp(ev->key, "KP_Down")))
+   else if ((!strcmp(ev->key, "Down")) || (!strcmp(ev->key, "KP_Down")))
      _e_menu_item_activate_next();
-   else if ((!strcmp(ev->key, "Left")) ||
-	    (!strcmp(ev->key, "KP_Left")))
+   else if ((!strcmp(ev->key, "Left")) || (!strcmp(ev->key, "KP_Left")))
      _e_menu_activate_previous();
-   else if ((!strcmp(ev->key, "Right")) ||
-	    (!strcmp(ev->key, "KP_Right")))
+   else if ((!strcmp(ev->key, "Right")) || (!strcmp(ev->key, "KP_Right")))
      _e_menu_activate_next();
-   else if ((!strcmp(ev->key, "Home")) ||
-	    (!strcmp(ev->key, "KP_Home")))
+   else if ((!strcmp(ev->key, "Home")) || (!strcmp(ev->key, "KP_Home")))
      _e_menu_item_activate_first();
-   else if ((!strcmp(ev->key, "End")) ||
-	    (!strcmp(ev->key, "KP_End")))
+   else if ((!strcmp(ev->key, "End")) || (!strcmp(ev->key, "KP_End")))
      _e_menu_item_activate_last();
    else if (!strcmp(ev->key, "space"))
-     {
-	_e_menu_active_call();
-     }
-   else if ((!strcmp(ev->key, "Return")) ||
-	    (!strcmp(ev->key, "KP_Enter")))
+     _e_menu_active_call();
+   else if ((!strcmp(ev->key, "Return")) || (!strcmp(ev->key, "KP_Enter")))
      {
 	_e_menu_active_call();
 	_e_menu_deactivate_all();
@@ -2612,17 +2635,17 @@ _e_menu_cb_key_down(void *data, int type, void *event)
      _e_menu_item_activate_last();
    else if (ev->compose)
      _e_menu_item_activate_char(ev->compose);
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
-_e_menu_cb_key_up(void *data, int type, void *event)
+static Eina_Bool
+_e_menu_cb_key_up(__UNUSED__ void *data, __UNUSED__ int type, void *event)
 {
    Ecore_Event_Key *ev;
    
    ev = event;
-   if (ev->window != _e_menu_win) return 1;
-   return 1;
+   if (ev->window != _e_menu_win) return ECORE_CALLBACK_PASS_ON;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
 /* we need all of these because menus are special and grab the mouse and
@@ -2630,13 +2653,13 @@ _e_menu_cb_key_up(void *data, int type, void *event)
  * events directly to the canvases from our grab window
  */
 
-static int
-_e_menu_cb_mouse_down(void *data, int type, void *event)
+static Eina_Bool
+_e_menu_cb_mouse_down(__UNUSED__ void *data, __UNUSED__ int type, void *event)
 {
    Ecore_Event_Mouse_Button *ev;
    
    ev = event;
-   if (ev->window != _e_menu_win) return 1;
+   if (ev->window != _e_menu_win) return ECORE_CALLBACK_PASS_ON;
 
    /* Only allow dragging from floating menus for now.
     * The reason for this is that for non floating menus, 
@@ -2646,25 +2669,25 @@ _e_menu_cb_mouse_down(void *data, int type, void *event)
    if (_e_menu_activate_floating)
      _e_menu_activate_maybe_drag = 1;
 
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
-_e_menu_cb_mouse_up(void *data, int type, void *event)
+static Eina_Bool
+_e_menu_cb_mouse_up(__UNUSED__ void *data, __UNUSED__ int type, void *event)
 {
    Ecore_Event_Mouse_Button *ev;
    Ecore_X_Time t;
    int ret = 0;
    
    ev = event;
-   if (ev->window != _e_menu_win) return 1;
+   if (ev->window != _e_menu_win) return ECORE_CALLBACK_PASS_ON;
 
    t = ev->timestamp - _e_menu_activate_time;
    if ((_e_menu_activate_time != 0) &&
        (t < (e_config->menus_click_drag_timeout * 1000)))
      {
 	_e_menu_activate_floating = 1;
-	return 1;
+	return ECORE_CALLBACK_PASS_ON;
      }
 
    if (_e_menu_activate_dragging)
@@ -2687,11 +2710,11 @@ _e_menu_cb_mouse_up(void *data, int type, void *event)
      _e_menu_deactivate_all();
    else if (!_e_menu_activate_floating)
      _e_menu_deactivate_all();
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
-_e_menu_cb_mouse_move(void *data, int type, void *event)
+static Eina_Bool
+_e_menu_cb_mouse_move(__UNUSED__ void *data, __UNUSED__ int type, void *event)
 {
    Ecore_Event_Mouse_Move *ev;
    Eina_List *l, *tmp;
@@ -2702,7 +2725,7 @@ _e_menu_cb_mouse_move(void *data, int type, void *event)
    int is_fast = 0;
    
    ev = event;
-   if (ev->window != _e_menu_win) return 1;
+   if (ev->window != _e_menu_win) return ECORE_CALLBACK_PASS_ON;
    fast_move_threshold = e_config->menus_fast_mouse_move_threshhold;
    dx = ev->x - _e_menu_x;
    dy = ev->y - _e_menu_y;
@@ -2735,8 +2758,7 @@ _e_menu_cb_mouse_move(void *data, int type, void *event)
 	     evas_event_feed_mouse_move(m->evas,
 					ev->x - m->cur.x + m->zone->x,
 					ev->y - m->cur.y + m->zone->y,
-					ev->timestamp,
-					NULL);
+					ev->timestamp, NULL);
 	  }
      }
 
@@ -2746,16 +2768,16 @@ _e_menu_cb_mouse_move(void *data, int type, void *event)
    _e_menu_y = ev->y;
    _e_menu_time = ev->timestamp;
    _e_menu_mouse_autoscroll_check();
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
-_e_menu_cb_mouse_wheel(void *data, int type, void *event)
+static Eina_Bool
+_e_menu_cb_mouse_wheel(__UNUSED__ void *data, __UNUSED__ int type, void *event)
 {
    Ecore_Event_Mouse_Wheel *ev;
    
    ev = event;
-   if (ev->window != _e_menu_win) return 1;
+   if (ev->window != _e_menu_win) return ECORE_CALLBACK_PASS_ON;
    if (ev->z < 0) /* up */
      {
 	int i;
@@ -2770,11 +2792,11 @@ _e_menu_cb_mouse_wheel(void *data, int type, void *event)
 	for (i = ev->z; i > 0; i--)
 	  _e_menu_item_activate_next();
      }
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
-_e_menu_cb_scroll_animator(void *data)
+static Eina_Bool
+_e_menu_cb_scroll_animator(__UNUSED__ void *data)
 {
    double t, dt;
    double dx, dy;
@@ -2823,8 +2845,8 @@ _e_menu_cb_scroll_animator(void *data)
    return 1;
 }
 
-static int
-_e_menu_cb_window_shape(void *data, int ev_type, void *ev)
+static Eina_Bool
+_e_menu_cb_window_shape(__UNUSED__ void *data, __UNUSED__ int ev_type, void *ev)
 {
    Eina_List *l;
    Ecore_X_Event_Window_Shape *e;
@@ -2836,7 +2858,7 @@ _e_menu_cb_window_shape(void *data, int ev_type, void *ev)
 	if (m->evas_win == e->win)
 	  m->need_shape_export = 1;
      }
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
 static void
@@ -2852,19 +2874,15 @@ _e_menu_cb_item_submenu_post_default(void *data, E_Menu *m, E_Menu_Item *mi)
 }
 
 
-static Evas_Bool
-_e_menu_categories_free_cb(const Eina_Hash *hash, const void *key, void *data, void *fdata)
+static Eina_Bool
+_e_menu_categories_free_cb(__UNUSED__ const Eina_Hash *hash, __UNUSED__ const void *key, void *data, __UNUSED__ void *fdata)
 {
-   Eina_List *l;
+   E_Menu_Category_Callback *cb;
    E_Menu_Category *cat;
 
    cat = (E_Menu_Category *) data;
-   l = (Eina_List *) cat->callbacks;
-   while (l)
-     {
-	free(l->data); /* free the callback struct */
-	l = eina_list_remove_list(l, l);
-     }
+   EINA_LIST_FREE(cat->callbacks, cb)
+     free(cb); /* free the callback struct */
    free(cat);
-   return 1;
+   return EINA_TRUE;
 }
