@@ -4,23 +4,9 @@
 #include "e.h"
 #include "e_mod_main.h"
 
-/***************************************************************************/
-/**/
 /* actual module specifics */
-
 static E_Module *conf_module = NULL;
 
-/**/
-/***************************************************************************/
-
-/***************************************************************************/
-/**/
-
-/**/
-/***************************************************************************/
-
-/***************************************************************************/
-/**/
 /* module setup */
 EAPI E_Module_Api e_modapi =
 {
@@ -44,8 +30,11 @@ EAPI int
 e_modapi_shutdown(E_Module *m)
 {
    E_Config_Dialog *cfd;
-   while ((cfd = e_config_dialog_get("E", "_config_border_style_dialog"))) e_object_del(E_OBJECT(cfd));
-   while ((cfd = e_config_dialog_get("E", "_config_border_border_style_dialog"))) e_object_del(E_OBJECT(cfd));
+
+   while ((cfd = e_config_dialog_get("E", "internal/borders_border"))) 
+     e_object_del(E_OBJECT(cfd));
+   while ((cfd = e_config_dialog_get("E", "appearance/borders"))) 
+     e_object_del(E_OBJECT(cfd));
    e_configure_registry_item_del("internal/borders_border");
    e_configure_registry_category_del("internal");
    e_configure_registry_item_del("appearance/borders");
