@@ -6,7 +6,7 @@
 #ifndef E_INT_CONFIG_FONTS_H
 #define E_INT_CONFIG_FONTS_H
 
-EAPI E_Config_Dialog *e_int_config_fonts(E_Container *con, const char *params __UNUSED__);
+E_Config_Dialog *e_int_config_fonts(E_Container *con, const char *params __UNUSED__);
 
 #endif
 #endif
