@@ -3,8 +3,8 @@
 #ifndef E_INT_CONFIG_IMC_IMPORT_H
 #define E_INT_CONFIG_IMC_IMPORT_H
 
-EAPI E_Win *e_int_config_imc_import(E_Config_Dialog *parent);
-EAPI void e_int_config_imc_import_del(E_Win *win);
+E_Win *e_int_config_imc_import(E_Config_Dialog *parent);
+void e_int_config_imc_import_del(E_Win *win);
     
 #endif
 #endif
