@@ -58,13 +58,13 @@ struct _E_Config_Dialog_Data
 #endif   
 };
 
-EAPI E_Config_Dialog *
+E_Config_Dialog *
 e_int_config_wallpaper(E_Container *con, const char *params __UNUSED__)
 {
    return _e_int_config_wallpaper_desk(con, -1, -1, -1, -1);
 }
 
-EAPI E_Config_Dialog *
+E_Config_Dialog *
 e_int_config_wallpaper_desk(E_Container *con, const char *params)
 {
    int con_num, zone_num, desk_x, desk_y;
@@ -83,7 +83,7 @@ _e_int_config_wallpaper_desk(E_Container *con, int con_num, int zone_num, int de
    E_Config_Dialog_View *v;
    E_Config_Wallpaper *cw;
 
-   if (e_config_dialog_find("E", "_config_wallpaper_dialog")) return NULL;
+   if (e_config_dialog_find("E", "appearance/wallpaper")) return NULL;
    v = E_NEW(E_Config_Dialog_View, 1);
    cw = E_NEW(E_Config_Wallpaper, 1);
 
@@ -108,12 +108,12 @@ _e_int_config_wallpaper_desk(E_Container *con, int con_num, int zone_num, int de
    cw->desk_y = desk_y;
 
    cfd = e_config_dialog_new(con, _("Wallpaper Settings"), "E", 
-			     "_config_wallpaper_dialog",
+			     "appearance/wallpaper",
 			     "preferences-desktop-wallpaper", 0, v, cw);
    return cfd;
 }
 
-EAPI void
+void
 e_int_config_wallpaper_update(E_Config_Dialog *dia, char *file)
 {
    E_Config_Dialog_Data *cfdata;
@@ -135,7 +135,7 @@ e_int_config_wallpaper_update(E_Config_Dialog *dia, char *file)
    if (cfdata->o_fm) e_widget_change(cfdata->o_fm);
 }
 
-EAPI void
+void
 e_int_config_wallpaper_import_done(E_Config_Dialog *dia)
 {
    E_Config_Dialog_Data *cfdata;
@@ -144,7 +144,7 @@ e_int_config_wallpaper_import_done(E_Config_Dialog *dia)
    cfdata->win_import = NULL;
 }
 
-EAPI void
+void
 e_int_config_wallpaper_gradient_done(E_Config_Dialog *dia)
 {
    E_Config_Dialog_Data *cfdata;
@@ -154,7 +154,7 @@ e_int_config_wallpaper_gradient_done(E_Config_Dialog *dia)
 }
 
 #ifdef HAVE_EXCHANGE
-EAPI void
+void
 e_int_config_wallpaper_web_done(E_Config_Dialog *dia)
 {
    E_Config_Dialog_Data *cfdata;
@@ -164,38 +164,6 @@ e_int_config_wallpaper_web_done(E_Config_Dialog *dia)
 }
 #endif
 
-EAPI void 
-e_int_config_wallpaper_handler_set(Evas_Object *obj, const char *path, void *data) 
-{
-   const char *dev, *fpath;
-
-   if (!path) return;
-   e_fm2_path_get(obj, &dev, &fpath);
-   if (e_config->wallpaper_import_last_dev)
-     {
-	eina_stringshare_del(e_config->wallpaper_import_last_dev);
-	e_config->wallpaper_import_last_dev = NULL;
-     }
-   if (dev)
-     e_config->wallpaper_import_last_dev = eina_stringshare_add(dev);
-   if (e_config->wallpaper_import_last_path) {
-     eina_stringshare_del(e_config->wallpaper_import_last_path);
-     e_config->wallpaper_import_last_path = NULL;
-   }
-   if (fpath)
-     e_config->wallpaper_import_last_path = eina_stringshare_add(fpath);
-   e_config_save_queue();
-
-   e_int_config_wallpaper_import(NULL, path);
-}
-
-EAPI int 
-e_int_config_wallpaper_handler_test(Evas_Object *obj, const char *path, void *data) 
-{
-   if (!path) return 0;
-   return 1;
-}
-
 static void
 _cb_button_up(void *data1, void *data2)
 {
@@ -538,7 +506,7 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cfdata)
 				  _cb_files_files_deleted, cfdata);
    e_widget_flist_path_set(ow, path, "/");
 
-   e_widget_min_size_set(ow, 160, 160);
+   e_widget_size_min_set(ow, 160, 160);
    e_widget_table_object_append(ot, ow, 0, 2, 1, 1, 1, 1, 1, 1);
    e_widget_list_object_append(o, ot, 1, 1, 0.0);
 
@@ -554,6 +522,7 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cfdata)
    ow = e_widget_button_add(evas, _("Gradient..."), "preferences-gradient",
 			    _cb_gradient, cfdata, NULL);
    e_widget_table_object_append(ot, ow, 1, 1, 1, 1, 1, 0, 0, 0);
+
 #ifdef HAVE_EXCHANGE
    if (online)
      {
@@ -574,6 +543,8 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cfdata)
      f = e_theme_edje_file_get("base/theme/backgrounds", "e/desktop/background");
    e_widget_preview_edje_set(ow, f, "e/desktop/background");
    e_widget_aspect_child_set(oa, ow);
+   evas_object_show(ow);
+   evas_object_show(oa);
    e_widget_table_object_append(ot, oa, 0, 2, 2 + online, 1, 1, 1, 1, 1);
    e_widget_list_object_append(o, ot, 1, 1, 0.5);
    e_dialog_resizable_set(cfd->dia, 1);
@@ -666,7 +637,7 @@ _adv_create(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cfdata)
                                   _cb_files_files_changed, cfdata);
    e_widget_flist_path_set(ow, path, "/");
 
-   e_widget_min_size_set(ow, 160, 160);
+   e_widget_size_min_set(ow, 160, 160);
    e_widget_table_object_append(ot, ow, 0, 2, 1, 1, 1, 1, 1, 1);
    e_widget_list_object_append(o, ot, 1, 1, 0.0);
 
@@ -682,6 +653,7 @@ _adv_create(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cfdata)
    ow = e_widget_button_add(evas, _("Gradient..."), "preferences-gradient",
 			    _cb_gradient, cfdata, NULL);
    e_widget_table_object_append(ot, ow, 1, 1, 1, 1, 1, 0, 0, 0);
+
 #ifdef HAVE_EXCHANGE
    if (online)
      {
