@@ -3,7 +3,7 @@
 #ifndef E_INT_CONFIG_REMEMBERS_H
 #define E_INT_CONFIG_REMEMBERS_H
 
-EAPI E_Config_Dialog *e_int_config_remembers(E_Container *con, const char *params __UNUSED__);
+E_Config_Dialog *e_int_config_remembers(E_Container *con, const char *params __UNUSED__);
 
 #endif
 #endif
