@@ -4,13 +4,11 @@
 #include "e.h"
 #include "e_mod_main.h"
 
-#ifdef __FreeBSD__   
+#ifdef __FreeBSD__
 #include <sys/types.h>
 #include <sys/sysctl.h>
-#endif  
+#endif
 
-/***************************************************************************/
-/**/
 /* gadcon requirements */
 static E_Gadcon_Client *_gc_init(E_Gadcon *gc, const char *name, const char *id, const char *style);
 static void _gc_shutdown(E_Gadcon_Client *gcc);
@@ -21,33 +19,27 @@ static const char *_gc_id_new(E_Gadcon_Client_Class *client_class);
 /* and actually define the gadcon class that this module provides (just 1) */
 static const E_Gadcon_Client_Class _gadcon_class =
 {
-   GADCON_CLIENT_CLASS_VERSION,
-     "cpufreq",
+   GADCON_CLIENT_CLASS_VERSION, "cpufreq",
      {
-        _gc_init, _gc_shutdown, _gc_orient, _gc_label, _gc_icon, _gc_id_new, NULL
+        _gc_init, _gc_shutdown, _gc_orient, _gc_label, _gc_icon, _gc_id_new, NULL, NULL
      },
    E_GADCON_CLIENT_STYLE_PLAIN
 };
-/**/
-/***************************************************************************/
 
-/***************************************************************************/
-/**/
 /* actual module specifics */
-
 typedef struct _Instance Instance;
 
 struct _Instance
 {
    E_Gadcon_Client *gcc;
-   Evas_Object     *o_cpu;
+   Evas_Object *o_cpu;
 };
 
 static void _button_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info);
 static void _menu_cb_post(void *data, E_Menu *m);
 static void _cpufreq_set_governor(const char *governor);
 static void _cpufreq_set_frequency(int frequency);
-static int _cpufreq_cb_check(void *data);
+static Eina_Bool _cpufreq_cb_check(void *data);
 static Status *_cpufreq_status_new();
 static void _cpufreq_status_free(Status *s);
 static int _cpufreq_status_check_available(Status *s);
@@ -57,7 +49,7 @@ static void _cpufreq_face_update_available(Instance *inst);
 static void _cpufreq_face_update_current(Instance *inst);
 static void _cpufreq_face_cb_set_frequency(void *data, Evas_Object *o, const char *emission, const char *source);
 static void _cpufreq_face_cb_set_governor(void *data, Evas_Object *o, const char *emission, const char *source);
-static int _cpufreq_event_cb_powersave(void *data __UNUSED__, int type, void *event);
+static Eina_Bool _cpufreq_event_cb_powersave(void *data __UNUSED__, int type, void *event);
 
 static void _cpufreq_menu_fast(void *data, E_Menu *m, E_Menu_Item *mi);
 static void _cpufreq_menu_medium(void *data, E_Menu *m, E_Menu_Item *mi);
@@ -80,32 +72,37 @@ _gc_init(E_Gadcon *gc, const char *name, const char *id, const char *style)
    Evas_Object *o;
    E_Gadcon_Client *gcc;
    Instance *inst;
-   
+
    inst = E_NEW(Instance, 1);
-   
+
    o = edje_object_add(gc->evas);
    e_theme_edje_object_set(o, "base/theme/modules/cpufreq",
 			   "e/modules/cpufreq/main");
-   edje_object_signal_callback_add(o, "e,action,governor,next", "*", _cpufreq_face_cb_set_governor, NULL);
-   edje_object_signal_callback_add(o, "e,action,frequency,increase", "*", _cpufreq_face_cb_set_frequency, NULL);
-   edje_object_signal_callback_add(o, "e,action,frequency,decrease", "*", _cpufreq_face_cb_set_frequency, NULL);
-   
+   edje_object_signal_callback_add(o, "e,action,governor,next", "*", 
+                                   _cpufreq_face_cb_set_governor, NULL);
+   edje_object_signal_callback_add(o, "e,action,frequency,increase", "*", 
+                                   _cpufreq_face_cb_set_frequency, NULL);
+   edje_object_signal_callback_add(o, "e,action,frequency,decrease", "*", 
+                                   _cpufreq_face_cb_set_frequency, NULL);
+
    gcc = e_gadcon_client_new(gc, name, id, style, o);
    gcc->data = inst;
-   
+
    inst->gcc = gcc;
    inst->o_cpu = o;
-   
+
    evas_object_event_callback_add(o, EVAS_CALLBACK_MOUSE_DOWN,
 				  _button_cb_mouse_down, inst);
-   cpufreq_config->instances = eina_list_append(cpufreq_config->instances, inst);
+   cpufreq_config->instances = 
+     eina_list_append(cpufreq_config->instances, inst);
    if (cpufreq_config->status) _cpufreq_status_free(cpufreq_config->status);
    cpufreq_config->status = _cpufreq_status_new();
    _cpufreq_cb_check(NULL);
    _cpufreq_face_update_available(inst);
 
-   cpufreq_config->handler = ecore_event_handler_add(E_EVENT_POWERSAVE_UPDATE,
-	_cpufreq_event_cb_powersave, NULL);
+   cpufreq_config->handler = 
+     ecore_event_handler_add(E_EVENT_POWERSAVE_UPDATE,
+                             _cpufreq_event_cb_powersave, NULL);
    return gcc;
 }
 
@@ -113,13 +110,15 @@ static void
 _gc_shutdown(E_Gadcon_Client *gcc)
 {
    Instance *inst;
-   
+
    inst = gcc->data;
-   cpufreq_config->instances = eina_list_remove(cpufreq_config->instances, inst);
+   cpufreq_config->instances = 
+     eina_list_remove(cpufreq_config->instances, inst);
    evas_object_del(inst->o_cpu);
    free(inst);
 
-   if (cpufreq_config->handler) ecore_event_handler_del(cpufreq_config->handler);
+   if (cpufreq_config->handler) 
+     ecore_event_handler_del(cpufreq_config->handler);
 }
 
 static void
@@ -139,8 +138,8 @@ static Evas_Object *
 _gc_icon(E_Gadcon_Client_Class *client_class, Evas *evas)
 {
    Evas_Object *o;
-   char buf[4096];
-   
+   char buf[PATH_MAX];
+
    o = edje_object_add(evas);
    snprintf(buf, sizeof(buf), "%s/e-module-cpufreq.edj",
 	    e_module_dir_get(cpufreq_config->module));
@@ -151,20 +150,19 @@ _gc_icon(E_Gadcon_Client_Class *client_class, Evas *evas)
 static const char *
 _gc_id_new(E_Gadcon_Client_Class *client_class)
 {
-   return _gadcon_class.name;
+   static char idbuff[32];
+
+   snprintf(idbuff, sizeof(idbuff), "%s.%d", _gadcon_class.name, 
+            eina_list_count(cpufreq_config->instances));
+   return idbuff;
 }
 
-/**/
-/***************************************************************************/
-
-/***************************************************************************/
-/**/
 static void
 _button_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
 {
    Instance *inst;
    Evas_Event_Mouse_Down *ev;
-   
+
    inst = data;
    ev = event_info;
    if ((ev->button == 3) && (!cpufreq_config->menu))
@@ -174,10 +172,10 @@ _button_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
 	int cx, cy;
 	Eina_List *l;
 	char buf[256];
-	
+
 	mn = e_menu_new();
 	cpufreq_config->menu_poll = mn;
-	
+
 	mi = e_menu_item_new(mn);
 	e_menu_item_label_set(mi, _("Fast (4 ticks)"));
 	e_menu_item_radio_set(mi, 1);
@@ -210,14 +208,15 @@ _button_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
 	e_menu_item_label_set(mi, _("Very Slow (256 ticks)"));
 	e_menu_item_radio_set(mi, 1);
 	e_menu_item_radio_group_set(mi, 1);
-	if (cpufreq_config->poll_interval >= 128) e_menu_item_toggle_set(mi, 1);
+	if (cpufreq_config->poll_interval >= 128) 
+          e_menu_item_toggle_set(mi, 1);
 	e_menu_item_callback_set(mi, _cpufreq_menu_very_slow, NULL);
 
 	if (cpufreq_config->status->governors)
 	  {
 	     mn = e_menu_new();
 	     cpufreq_config->menu_governor = mn;
-	     
+
 	     for (l = cpufreq_config->status->governors; l; l = l->next)
 	       {
 		  mi = e_menu_item_new(mn);
@@ -269,7 +268,7 @@ _button_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
 		  e_menu_item_radio_set(mi, 1);
 		  e_menu_item_radio_group_set(mi, 1);
 		  if (cpufreq_config->powersave_governor
-		       	&& !strcmp(cpufreq_config->powersave_governor, l->data))
+		      && !strcmp(cpufreq_config->powersave_governor, l->data))
 		    e_menu_item_toggle_set(mi, 1);
 		  e_menu_item_callback_set(mi, _cpufreq_menu_powersave_governor, l->data);
 	       }
@@ -288,17 +287,17 @@ _button_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
 	  {
 	     mn = e_menu_new();
 	     cpufreq_config->menu_frequency = mn;
-	
+
 	     for (l = cpufreq_config->status->frequencies; l; l = l->next)
 	       {
 		  int frequency;
-		  
-		  frequency = (int)l->data;
+
+		  frequency = (long)l->data;
 		  mi = e_menu_item_new(mn);
 		  if (frequency < 1000000)
 		    snprintf(buf, sizeof(buf), _("%i MHz"), frequency / 1000);
 		  else
-		    snprintf(buf, sizeof(buf), _("%i.%i GHz"), 
+		    snprintf(buf, sizeof(buf), _("%i.%i GHz"),
 			     frequency / 1000000, (frequency % 1000000) / 100000);
 		  buf[sizeof(buf) - 1] = 0;
 		  e_menu_item_label_set(mi, buf);
@@ -309,11 +308,11 @@ _button_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
 		  e_menu_item_callback_set(mi, _cpufreq_menu_frequency, l->data);
 	       }
 	  }
-	
+
 	mn = e_menu_new();
 	cpufreq_config->menu = mn;
 	e_menu_post_deactivate_callback_set(mn, _menu_cb_post, inst);
-	
+
 	mi = e_menu_item_new(mn);
 	e_menu_item_label_set(mi, _("Time Between Updates"));
 	e_menu_item_submenu_set(mi, cpufreq_config->menu_poll);
@@ -324,7 +323,7 @@ _button_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
 	     e_menu_item_label_set(mi, _("Set CPU Power Policy"));
 	     e_menu_item_submenu_set(mi, cpufreq_config->menu_governor);
 	  }
-	
+
 	if (cpufreq_config->menu_frequency)
 	  {
 	     mi = e_menu_item_new(mn);
@@ -337,17 +336,16 @@ _button_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
 	     e_menu_item_label_set(mi, _("Powersaving behavior"));
 	     e_menu_item_submenu_set(mi, cpufreq_config->menu_powersave);
 	  }
-	
+
         e_gadcon_client_util_menu_items_append(inst->gcc, mn, 0);
-	
+
 	e_gadcon_canvas_zone_geometry_get(inst->gcc->gadcon,
 					  &cx, &cy, NULL, NULL);
 	e_menu_activate_mouse(mn,
 			      e_util_zone_current_get(e_manager_current_get()),
 			      cx + ev->output.x, cy + ev->output.y, 1, 1,
-			      E_MENU_POP_DIRECTION_DOWN, ev->timestamp);
+			      E_MENU_POP_DIRECTION_AUTO, ev->timestamp);
      }
-
 }
 
 static void
@@ -356,31 +354,37 @@ _menu_cb_post(void *data, E_Menu *m)
    if (!cpufreq_config->menu) return;
    e_object_del(E_OBJECT(cpufreq_config->menu));
    cpufreq_config->menu = NULL;
-   if (cpufreq_config->menu_poll) e_object_del(E_OBJECT(cpufreq_config->menu_poll));
+   if (cpufreq_config->menu_poll) 
+     e_object_del(E_OBJECT(cpufreq_config->menu_poll));
    cpufreq_config->menu_poll = NULL;
-   if (cpufreq_config->menu_governor) e_object_del(E_OBJECT(cpufreq_config->menu_governor));
+   if (cpufreq_config->menu_governor) 
+     e_object_del(E_OBJECT(cpufreq_config->menu_governor));
    cpufreq_config->menu_governor = NULL;
-   if (cpufreq_config->menu_frequency) e_object_del(E_OBJECT(cpufreq_config->menu_frequency));
+   if (cpufreq_config->menu_frequency) 
+     e_object_del(E_OBJECT(cpufreq_config->menu_frequency));
    cpufreq_config->menu_frequency = NULL;
-   if (cpufreq_config->menu_powersave) e_object_del(E_OBJECT(cpufreq_config->menu_powersave));
+   if (cpufreq_config->menu_powersave) 
+     e_object_del(E_OBJECT(cpufreq_config->menu_powersave));
    cpufreq_config->menu_powersave = NULL;
 }
 
 static void
 _cpufreq_set_governor(const char *governor)
 {
-   char buf[4096];
+   char buf[PATH_MAX];
    int ret;
 
-   snprintf(buf, sizeof(buf), 
+   snprintf(buf, sizeof(buf),
 	    "%s %s %s", cpufreq_config->set_exe_path, "governor", governor);
    ret = system(buf);
    if (ret != 0)
      {
 	E_Dialog *dia;
+        E_Container *con;
 
-	dia = e_dialog_new(e_container_current_get(e_manager_current_get()), "E", "_e_mod_cpufreq_error_setfreq");
-	if (!dia) return;
+        con = e_container_current_get(e_manager_current_get());
+	if (!(dia = e_dialog_new(con, "E", "_e_mod_cpufreq_error_setfreq")))
+          return;
 	e_dialog_title_set(dia, "Enlightenment Cpufreq Module");
 	e_dialog_icon_set(dia, "enlightenment", 64);
 	e_dialog_text_set(dia, _("There was an error trying to set the<br>"
@@ -395,18 +399,20 @@ _cpufreq_set_governor(const char *governor)
 static void
 _cpufreq_set_frequency(int frequency)
 {
-   char buf[4096];
+   char buf[PATH_MAX];
    int ret;
-   
+
 #ifdef __FreeBSD__
    frequency /= 1000;
-#endif 
+#endif
    if (!cpufreq_config->status->can_set_frequency)
      {
 	E_Dialog *dia;
+        E_Container *con;
 
-	dia = e_dialog_new(e_container_current_get(e_manager_current_get()), "E", "_e_mod_cpufreq_error_setfreq");
-	if (!dia) return;
+        con = e_container_current_get(e_manager_current_get());
+	if (!(dia = e_dialog_new(con, "E", "_e_mod_cpufreq_error_setfreq")))
+          return;
 	e_dialog_title_set(dia, "Enlightenment Cpufreq Module");
 	e_dialog_icon_set(dia, "enlightenment", 64);
 	e_dialog_text_set(dia, _("Your kernel does not support setting the<br>"
@@ -418,15 +424,21 @@ _cpufreq_set_frequency(int frequency)
 	e_dialog_show(dia);
 	return;
      }
+
+   // change it to "userspace"
+   _cpufreq_set_governor("userspace");
+
    snprintf(buf, sizeof(buf),
 	    "%s %s %i", cpufreq_config->set_exe_path, "frequency", frequency);
    ret = system(buf);
    if (ret != 0)
      {
 	E_Dialog *dia;
+        E_Container *con;
 
-	dia = e_dialog_new(e_container_current_get(e_manager_current_get()), "E", "_e_mod_cpufreq_error_setfreq");
-	if (!dia) return;
+        con = e_container_current_get(e_manager_current_get());
+	if (!(dia = e_dialog_new(con, "E", "_e_mod_cpufreq_error_setfreq")))
+          return;
 	e_dialog_title_set(dia, "Enlightenment Cpufreq Module");
 	e_dialog_icon_set(dia, "enlightenment", 64);
 	e_dialog_text_set(dia, _("There was an error trying to set the<br>"
@@ -438,18 +450,18 @@ _cpufreq_set_frequency(int frequency)
      }
 }
 
-static int
+static Eina_Bool
 _cpufreq_cb_check(void *data)
 {
    Instance *inst;
    Eina_List *l;
    int active;
 
-   if (cpufreq_config->menu_poll) return 1;
+   if (cpufreq_config->menu_poll) return ECORE_CALLBACK_RENEW;
    active = cpufreq_config->status->active;
    if (_cpufreq_status_check_current(cpufreq_config->status))
      {
-	for (l = cpufreq_config->instances; l; l = l->next) 
+	for (l = cpufreq_config->instances; l; l = l->next)
 	  {
 	     inst = l->data;
 	     _cpufreq_face_update_current(inst);
@@ -457,7 +469,7 @@ _cpufreq_cb_check(void *data)
      }
    if (active != cpufreq_config->status->active)
      {
-	for (l = cpufreq_config->instances; l; l = l->next) 
+	for (l = cpufreq_config->instances; l; l = l->next)
 	  {
 	     inst = l->data;
 	     if (cpufreq_config->status->active == 0)
@@ -467,7 +479,7 @@ _cpufreq_cb_check(void *data)
 	  }
      }
 
-   return 1;
+   return ECORE_CALLBACK_RENEW;
 }
 
 static Status *
@@ -502,8 +514,8 @@ _cpufreq_cb_sort(const void *item1, const void *item2)
 {
    int a, b;
 
-   a = (int)item1;
-   b = (int)item2;
+   a = (long)item1;
+   b = (long)item2;
    if (a < b) return -1;
    else if (a > b) return 1;
    return 0;
@@ -512,7 +524,7 @@ _cpufreq_cb_sort(const void *item1, const void *item2)
 static int
 _cpufreq_status_check_available(Status *s)
 {
-   char buf[4096];
+   char buf[PATH_MAX];
    Eina_List *l;
 #ifdef __FreeBSD__
    int freq, i;
@@ -522,7 +534,7 @@ _cpufreq_status_check_available(Status *s)
    /* read freq_levels sysctl and store it in freq */
    len = sizeof(buf);
    if (sysctlbyname("dev.cpu.0.freq_levels", buf, &len, NULL, 0) == 0)
-     {    
+     {
 	/* sysctl returns 0 on success */
 	if (s->frequencies)
 	  {
@@ -530,7 +542,7 @@ _cpufreq_status_check_available(Status *s)
 	     s->frequencies = NULL;
 	  }
 
-	/* parse freqs and store the frequencies in s->frequencies */ 
+	/* parse freqs and store the frequencies in s->frequencies */
 	pos = buf;
 	while (pos)
 	  {
@@ -546,7 +558,7 @@ _cpufreq_status_check_available(Status *s)
 	     pos = strchr(pos, ' ');
 	  }
      }
-   
+
    /* sort is not necessary because freq_levels is already sorted */
    /* freebsd doesn't have governors */
    if (s->governors)
@@ -557,29 +569,29 @@ _cpufreq_status_check_available(Status *s)
      }
 #else
    FILE *f;
-   
+
    f = fopen("/sys/devices/system/cpu/cpu0/cpufreq/scaling_available_frequencies", "r");
    if (f)
      {
 	char *freq;
-	
+
 	if (s->frequencies)
 	  {
 	     eina_list_free(s->frequencies);
 	     s->frequencies = NULL;
 	  }
-	
+
 	fgets(buf, sizeof(buf), f);
 	buf[sizeof(buf) - 1] = 0;
 	fclose(f);
-	
+
 	freq = strtok(buf, " ");
-	do 
+	do
 	  {
-	     if (atoi(freq) != 0) 
+	     if (atoi(freq) != 0)
 	       {
-		  s->frequencies = eina_list_append(s->frequencies,	
-						    (void *) atoi(freq));
+		  s->frequencies = eina_list_append(s->frequencies,
+						    (void *) (long)atoi(freq));
 	       }
 	     freq = strtok(NULL, " ");
 	  }
@@ -594,7 +606,7 @@ _cpufreq_status_check_available(Status *s)
    if (f)
      {
 	char *gov;
-	
+
 	if (s->governors)
 	  {
 	     for (l = s->governors; l; l = l->next)
@@ -608,7 +620,7 @@ _cpufreq_status_check_available(Status *s)
 	fclose(f);
 
 	gov = strtok(buf, " ");
-	do 
+	do
 	  {
 	     while ((*gov) && (isspace(*gov))) gov++;
 	     if (strlen(gov) != 0)
@@ -617,9 +629,9 @@ _cpufreq_status_check_available(Status *s)
 	  }
 	while (gov != NULL);
 
-	s->governors = eina_list_sort(s->governors,
-				      eina_list_count(s->governors),
-				      (int (*)(const void *, const void *))strcmp);
+	s->governors = 
+          eina_list_sort(s->governors, eina_list_count(s->governors),
+                         (int (*)(const void *, const void *))strcmp);
      }
 #endif
    return 1;
@@ -628,7 +640,7 @@ _cpufreq_status_check_available(Status *s)
 static int
 _cpufreq_status_check_current(Status *s)
 {
-   char buf[4096];
+   char buf[PATH_MAX];
    int i;
    FILE *f;
    int ret = 0;
@@ -646,12 +658,12 @@ _cpufreq_status_check_current(Status *s)
 	s->active = 1;
      }
 
-    /* hardcoded for testing */
-    s->can_set_frequency = 1;
-    s->cur_governor = NULL;
+   /* hardcoded for testing */
+   s->can_set_frequency = 1;
+   s->cur_governor = NULL;
 #else
    s->active = 0;
-   
+
    _cpufreq_status_check_available(s);
    f = fopen("/sys/devices/system/cpu/cpu0/cpufreq/scaling_cur_freq", "r");
    if (f)
@@ -659,14 +671,14 @@ _cpufreq_status_check_current(Status *s)
 	fgets(buf, sizeof(buf), f);
 	buf[sizeof(buf) - 1] = 0;
 	fclose(f);
-	
+
 	frequency = atoi(buf);
 	if (frequency != s->cur_frequency)
 	  ret = 1;
 	s->cur_frequency = frequency;
 	s->active = 1;
      }
-   
+
    f = fopen("/sys/devices/system/cpu/cpu0/cpufreq/scaling_setspeed", "r");
    if (f)
      {
@@ -677,7 +689,7 @@ _cpufreq_status_check_current(Status *s)
      {
 	s->can_set_frequency = 0;
      }
-   
+
    f = fopen("/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor", "r");
    if (f)
      {
@@ -718,8 +730,8 @@ _cpufreq_face_update_available(Instance *inst)
    count = eina_list_count(cpufreq_config->status->frequencies);
    frequency_msg = malloc(sizeof(Edje_Message_Int_Set) + (count - 1) * sizeof(int));
    frequency_msg->count = count;
-   for (l = cpufreq_config->status->frequencies, i = 0; l; l = l->next, i++) 
-     frequency_msg->val[i] = (int)l->data;
+   for (l = cpufreq_config->status->frequencies, i = 0; l; l = l->next, i++)
+     frequency_msg->val[i] = (long)l->data;
    edje_object_message_send(inst->o_cpu, EDJE_MESSAGE_INT_SET, 1, frequency_msg);
    free(frequency_msg);
 
@@ -751,7 +763,7 @@ _cpufreq_face_update_current(Instance *inst)
    if (cpufreq_config->status->cur_governor != NULL)
      {
 	governor_msg.str = cpufreq_config->status->cur_governor;
-	edje_object_message_send(inst->o_cpu, EDJE_MESSAGE_STRING, 4, 
+	edje_object_message_send(inst->o_cpu, EDJE_MESSAGE_STRING, 4,
 				 &governor_msg);
      }
 }
@@ -764,16 +776,16 @@ _cpufreq_face_cb_set_frequency(void *data, Evas_Object *obj, const char *emissio
 
    for (l = cpufreq_config->status->frequencies; l; l = l->next)
      {
-	if (cpufreq_config->status->cur_frequency == (int)l->data)
+	if (cpufreq_config->status->cur_frequency == (long)l->data)
 	  {
 	     if (!strcmp(emission, "e,action,frequency,increase"))
 	       {
-		  if (l->next) next_frequency = (int)l->next->data;
+		  if (l->next) next_frequency = (long)l->next->data;
 		  break;
 	       }
 	     else if (!strcmp(emission, "e,action,frequency,decrease"))
 	       {
-		  if (l->prev) next_frequency = (int)l->prev->data;
+		  if (l->prev) next_frequency = (long)l->prev->data;
 		  break;
 	       }
 	     else
@@ -803,7 +815,7 @@ _cpufreq_face_cb_set_governor(void *data, Evas_Object *obj, const char *emission
    if (next_governor != NULL) _cpufreq_set_governor(next_governor);
 }
 
-static int
+static Eina_Bool
 _cpufreq_event_cb_powersave(void *data __UNUSED__, int type, void *event)
 {
    E_Event_Powersave_Update *ev;
@@ -811,8 +823,8 @@ _cpufreq_event_cb_powersave(void *data __UNUSED__, int type, void *event)
    Eina_Bool has_powersave = EINA_FALSE;
    Eina_Bool has_conservative = EINA_FALSE;
 
-   if (type != E_EVENT_POWERSAVE_UPDATE) return 1;
-   if (!cpufreq_config->auto_powersave) return 1;
+   if (type != E_EVENT_POWERSAVE_UPDATE) return ECORE_CALLBACK_PASS_ON;
+   if (!cpufreq_config->auto_powersave) return ECORE_CALLBACK_PASS_ON;
 
    ev = event;
    if (!cpufreq_config->status->orig_governor)
@@ -836,7 +848,7 @@ _cpufreq_event_cb_powersave(void *data __UNUSED__, int type, void *event)
 	 break;
       case E_POWERSAVE_MODE_MEDIUM:
       case E_POWERSAVE_MODE_HIGH:
-	 if (cpufreq_config->powersave_governor || has_conservative)
+	 if ((cpufreq_config->powersave_governor) || (has_conservative))
 	   {
 	      if (cpufreq_config->powersave_governor)
 		_cpufreq_set_governor(cpufreq_config->powersave_governor);
@@ -850,7 +862,7 @@ _cpufreq_event_cb_powersave(void *data __UNUSED__, int type, void *event)
 	 break;
      }
 
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
 static void
@@ -858,7 +870,7 @@ _cpufreq_menu_fast(void *data, E_Menu *m, E_Menu_Item *mi)
 {
    cpufreq_config->poll_interval = 4;
    ecore_poller_del(cpufreq_config->frequency_check_poller);
-   cpufreq_config->frequency_check_poller = 
+   cpufreq_config->frequency_check_poller =
      ecore_poller_add(ECORE_POLLER_CORE, cpufreq_config->poll_interval,
 		      _cpufreq_cb_check, NULL);
    e_config_save_queue();
@@ -869,7 +881,7 @@ _cpufreq_menu_medium(void *data, E_Menu *m, E_Menu_Item *mi)
 {
    cpufreq_config->poll_interval = 8;
    ecore_poller_del(cpufreq_config->frequency_check_poller);
-   cpufreq_config->frequency_check_poller = 
+   cpufreq_config->frequency_check_poller =
      ecore_poller_add(ECORE_POLLER_CORE, cpufreq_config->poll_interval,
 		      _cpufreq_cb_check, NULL);
    e_config_save_queue();
@@ -880,7 +892,7 @@ _cpufreq_menu_normal(void *data, E_Menu *m, E_Menu_Item *mi)
 {
    cpufreq_config->poll_interval = 32;
    ecore_poller_del(cpufreq_config->frequency_check_poller);
-   cpufreq_config->frequency_check_poller = 
+   cpufreq_config->frequency_check_poller =
      ecore_poller_add(ECORE_POLLER_CORE, cpufreq_config->poll_interval,
 		      _cpufreq_cb_check, NULL);
    e_config_save_queue();
@@ -891,7 +903,7 @@ _cpufreq_menu_slow(void *data, E_Menu *m, E_Menu_Item *mi)
 {
    cpufreq_config->poll_interval = 64;
    ecore_poller_del(cpufreq_config->frequency_check_poller);
-   cpufreq_config->frequency_check_poller = 
+   cpufreq_config->frequency_check_poller =
      ecore_poller_add(ECORE_POLLER_CORE, cpufreq_config->poll_interval,
 		      _cpufreq_cb_check, NULL);
    e_config_save_queue();
@@ -902,7 +914,7 @@ _cpufreq_menu_very_slow(void *data, E_Menu *m, E_Menu_Item *mi)
 {
    cpufreq_config->poll_interval = 256;
    ecore_poller_del(cpufreq_config->frequency_check_poller);
-   cpufreq_config->frequency_check_poller = 
+   cpufreq_config->frequency_check_poller =
      ecore_poller_add(ECORE_POLLER_CORE, cpufreq_config->poll_interval,
 		      _cpufreq_cb_check, NULL);
    e_config_save_queue();
@@ -912,11 +924,13 @@ static void
 _cpufreq_menu_restore_governor(void *data, E_Menu *m, E_Menu_Item *mi)
 {
    cpufreq_config->restore_governor = e_menu_item_toggle_get(mi);
-   if ((!cpufreq_config->governor) || 
+   if ((!cpufreq_config->governor) ||
        (strcmp(cpufreq_config->status->cur_governor, cpufreq_config->governor)))
      {
-	if (cpufreq_config->governor) eina_stringshare_del(cpufreq_config->governor);
-	cpufreq_config->governor = eina_stringshare_add(cpufreq_config->status->cur_governor);
+	if (cpufreq_config->governor) 
+          eina_stringshare_del(cpufreq_config->governor);
+	cpufreq_config->governor = 
+          eina_stringshare_add(cpufreq_config->status->cur_governor);
      }
    e_config_save_queue();
 }
@@ -937,7 +951,8 @@ _cpufreq_menu_governor(void *data, E_Menu *m, E_Menu_Item *mi)
    if (governor)
      {
 	_cpufreq_set_governor(governor);
-	if (cpufreq_config->governor) eina_stringshare_del(cpufreq_config->governor);
+	if (cpufreq_config->governor) 
+          eina_stringshare_del(cpufreq_config->governor);
 	cpufreq_config->governor = eina_stringshare_add(governor);
      }
    e_config_save_queue();
@@ -962,26 +977,23 @@ static void
 _cpufreq_menu_frequency(void * data, E_Menu *m, E_Menu_Item *mi)
 {
    int frequency;
-   
-   frequency = (int)data;
+
+   frequency = (long)data;
    if (frequency > 0) _cpufreq_set_frequency(frequency);
 }
 
-/***************************************************************************/
-/**/
 /* module setup */
-EAPI E_Module_Api e_modapi = 
+EAPI E_Module_Api e_modapi =
 {
-   E_MODULE_API_VERSION,
-     "Cpufreq"
+   E_MODULE_API_VERSION, "Cpufreq"
 };
 
 EAPI void *
 e_modapi_init(E_Module *m)
 {
-   char buf[4096];
+   char buf[PATH_MAX];
    Eina_List *l;
-   
+
    conf_edd = E_CONFIG_DD_NEW("Cpufreq_Config", Config);
 #undef T
 #undef D
@@ -993,9 +1005,10 @@ e_modapi_init(E_Module *m)
    E_CONFIG_VAL(D, T, auto_powersave, INT);
    E_CONFIG_VAL(D, T, powersave_governor, STR);
    E_CONFIG_VAL(D, T, governor, STR);
-   
+
    cpufreq_config = e_config_domain_load("module.cpufreq", conf_edd);
-   if (cpufreq_config && cpufreq_config->config_version != CPUFREQ_CONFIG_VERSION)
+   if ((cpufreq_config) && 
+       (cpufreq_config->config_version != CPUFREQ_CONFIG_VERSION))
      E_FREE(cpufreq_config);
 
    if (!cpufreq_config)
@@ -1009,14 +1022,13 @@ e_modapi_init(E_Module *m)
 	cpufreq_config->governor = NULL;
      }
    E_CONFIG_LIMIT(cpufreq_config->poll_interval, 1, 1024);
-   
-   snprintf(buf, sizeof(buf), "%s/%s/freqset",
-	    e_module_dir_get(m), MODULE_ARCH);
+
+   snprintf(buf, sizeof(buf), "%s/%s/freqset", 
+            e_module_dir_get(m), MODULE_ARCH);
    cpufreq_config->set_exe_path = strdup(buf);
-   cpufreq_config->frequency_check_poller = 
-     ecore_poller_add(ECORE_POLLER_CORE,
-		     cpufreq_config->poll_interval,
-		     _cpufreq_cb_check, NULL);
+   cpufreq_config->frequency_check_poller =
+     ecore_poller_add(ECORE_POLLER_CORE, cpufreq_config->poll_interval,
+		      _cpufreq_cb_check, NULL);
    cpufreq_config->status = _cpufreq_status_new();
 
    _cpufreq_status_check_available(cpufreq_config->status);
@@ -1043,7 +1055,7 @@ EAPI int
 e_modapi_shutdown(E_Module *m)
 {
    e_gadcon_provider_unregister(&_gadcon_class);
-   
+
    if (cpufreq_config->frequency_check_poller)
      ecore_poller_del(cpufreq_config->frequency_check_poller);
    if (cpufreq_config->menu)
@@ -1092,5 +1104,3 @@ e_modapi_save(E_Module *m)
    e_config_domain_save("module.cpufreq", conf_edd, cpufreq_config);
    return 1;
 }
-/**/
-/***************************************************************************/
