@@ -42,6 +42,7 @@ e_cfg_init(E_Module *m)
    E_CONFIG_VAL(conf_edd, Illume_Cfg, launcher.mode, INT);
    E_CONFIG_VAL(conf_edd, Illume_Cfg, launcher.icon_size, INT);
    E_CONFIG_VAL(conf_edd, Illume_Cfg, launcher.single_click, INT);
+   E_CONFIG_VAL(conf_edd, Illume_Cfg, launcher.single_click_delay, INT);
    
    E_CONFIG_VAL(conf_edd, Illume_Cfg, power.auto_suspend, INT);
    E_CONFIG_VAL(conf_edd, Illume_Cfg, power.auto_suspend_delay, INT);
@@ -80,6 +81,7 @@ e_cfg_init(E_Module *m)
 	illume_cfg->launcher.mode = 0;
 	illume_cfg->launcher.icon_size = 120;
 	illume_cfg->launcher.single_click = 1;
+	illume_cfg->launcher.single_click_delay = 150;
 	
 	illume_cfg->power.auto_suspend = 1;
 	illume_cfg->power.auto_suspend_delay = 1;
@@ -159,10 +161,13 @@ e_cfg_save(void)
 
 ///////////////////////////////////////////////////////////////////////////////
 Ecore_Timer *_e_cfg_launcher_change_timer = NULL;
-static int _e_cfg_launcher_change_timeout(void *data) {
+static Eina_Bool
+_e_cfg_launcher_change_timeout(__UNUSED__ void *data)
+{
    _e_mod_win_cfg_update();
    e_config_save_queue();
-   _e_cfg_launcher_change_timer = NULL; return 0;
+   _e_cfg_launcher_change_timer = NULL;
+   return ECORE_CALLBACK_CANCEL;
 }
 static void
 _e_cfg_launcher_change(void *data, Evas_Object *obj, void *event_info) {
@@ -170,6 +175,14 @@ _e_cfg_launcher_change(void *data, Evas_Object *obj, void *event_info) {
    _e_cfg_launcher_change_timer = ecore_timer_add(0.5, _e_cfg_launcher_change_timeout, data);
 }
 
+Evas_Object *delay_label, *delay_slider;
+static void
+_e_cfg_launcher_click_change(void *data, Evas_Object *obj, void *event_info) {
+	e_widget_disabled_set(delay_label, !illume_cfg->launcher.single_click);
+	e_widget_disabled_set(delay_slider, !illume_cfg->launcher.single_click);
+	_e_cfg_launcher_change(data, obj, event_info);
+}
+
 static void *
 _e_cfg_launcher_create(E_Config_Dialog *cfd)
 { // alloc cfd->cfdata
@@ -179,6 +192,7 @@ _e_cfg_launcher_create(E_Config_Dialog *cfd)
 static void 
 _e_cfg_launcher_free(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata)
 { // free cfd->cfdata
+   _e_mod_win_cfg_update(); // Reload on exit (to apply the slider value)
 }
 
 static Evas_Object *
@@ -221,7 +235,16 @@ _e_cfg_launcher_ui(E_Config_Dialog *cfd, Evas *e, E_Config_Dialog_Data *cfdata)
    frame = e_widget_framelist_add(e, "Launch Action", 0);
    o = e_widget_check_add(e, "Single press", &(illume_cfg->launcher.single_click));
    e_widget_framelist_object_append(frame, o);
-   evas_object_smart_callback_add(o, "changed", _e_cfg_launcher_change, NULL);
+   evas_object_smart_callback_add(o, "changed", _e_cfg_launcher_click_change, NULL);
+   o = e_widget_label_add(e, "Press delay:");
+   delay_label = o;
+   e_widget_disabled_set(o, !illume_cfg->launcher.single_click);
+   e_widget_framelist_object_append(frame, o);
+   o = e_widget_slider_add(e, 1, 0, "%1.0f ms", 0, 350, 1, 0, NULL, &(illume_cfg->launcher.single_click_delay), 150);
+   delay_slider = o;
+   //evas_object_smart_callback_add(o, "changed", _e_cfg_launcher_change, NULL); //works ??
+   e_widget_disabled_set(o, !illume_cfg->launcher.single_click);
+   e_widget_framelist_object_append(frame, o);
    e_widget_list_object_append(list, frame, 1, 0, 0.0); // fill, expand, align
 
    return list;
@@ -249,7 +272,9 @@ e_cfg_launcher(E_Container *con, const char *params)
 
 ///////////////////////////////////////////////////////////////////////////////
 Ecore_Timer *_e_cfg_power_change_timer = NULL;
-static int _e_cfg_power_change_timeout(void *data) {
+static Eina_Bool
+_e_cfg_power_change_timeout(void *data)
+{
    if (e_config->screensaver_timeout > 0)
      {
 	e_config->screensaver_enable = 1;
@@ -270,7 +295,8 @@ static int _e_cfg_power_change_timeout(void *data) {
      }
    e_pwr_cfg_update();
    e_config_save_queue();
-   _e_cfg_power_change_timer = NULL; return 0;
+   _e_cfg_power_change_timer = NULL;
+   return ECORE_CALLBACK_CANCEL;
 }
 static void
 _e_cfg_power_change(void *data, Evas_Object *obj, void *event_info) {
@@ -372,9 +398,12 @@ e_cfg_power(E_Container *con, const char *params)
 
 ///////////////////////////////////////////////////////////////////////////////
 Ecore_Timer *_e_cfg_animation_change_timer = NULL;
-static int _e_cfg_animation_change_timeout(void *data) {
+static Eina_Bool
+_e_cfg_animation_change_timeout(__UNUSED__ void *data)
+{
    e_config_save_queue();
-   _e_cfg_animation_change_timer = NULL; return 0;
+   _e_cfg_animation_change_timer = NULL;
+   return ECORE_CALLBACK_CANCEL;
 }
 static void
 _e_cfg_animation_change(void *data, Evas_Object *obj, void *event_info) {
@@ -502,10 +531,13 @@ e_cfg_animation(E_Container *con, const char *params)
 
 ///////////////////////////////////////////////////////////////////////////////
 Ecore_Timer *_e_cfg_slipshelf_change_timer = NULL;
-static int _e_cfg_slipshelf_change_timeout(void *data) {
+static Eina_Bool
+_e_cfg_slipshelf_change_timeout(__UNUSED__ void *data)
+{
    _e_mod_win_slipshelf_cfg_update();
    e_config_save_queue();
-   _e_cfg_slipshelf_change_timer = NULL; return 0;
+   _e_cfg_slipshelf_change_timer = NULL;
+   return ECORE_CALLBACK_CANCEL;
 }
 static void
 _e_cfg_slipshelf_change(void *data, Evas_Object *obj, void *event_info) {
@@ -619,7 +651,9 @@ e_cfg_slipshelf(E_Container *con, const char *params)
 
 ///////////////////////////////////////////////////////////////////////////////
 Ecore_Timer *_e_cfg_thumbscroll_change_timer = NULL;
-static int _e_cfg_thumbscroll_change_timeout(void *data) {
+static Eina_Bool
+_e_cfg_thumbscroll_change_timeout(__UNUSED__ void *data)
+{
    if (e_config->thumbscroll_threshhold == 0)
      {
 	e_config->thumbscroll_enable = 0;
@@ -631,7 +665,8 @@ static int _e_cfg_thumbscroll_change_timeout(void *data) {
 	e_config->thumbscroll_friction = 1.0;
      }
    e_config_save_queue();
-   _e_cfg_thumbscroll_change_timer = NULL; return 0;
+   _e_cfg_thumbscroll_change_timer = NULL;
+   return ECORE_CALLBACK_CANCEL;
 }
 static void
 _e_cfg_thumbscroll_change(void *data, Evas_Object *obj, void *event_info) {
@@ -708,11 +743,14 @@ e_cfg_thumbscroll(E_Container *con, const char *params)
 
 ///////////////////////////////////////////////////////////////////////////////
 Ecore_Timer *_e_cfg_fps_change_timer = NULL;
-static int _e_cfg_fps_change_timeout(void *data) {
+static Eina_Bool
+_e_cfg_fps_change_timeout(__UNUSED__ void *data)
+{
    e_config->framerate = illume_cfg->performance.fps;
    edje_frametime_set(1.0 / e_config->framerate);
    e_config_save_queue();
-   _e_cfg_fps_change_timer = NULL; return 0;
+   _e_cfg_fps_change_timer = NULL;
+   return ECORE_CALLBACK_CANCEL;
 }
 static void
 _e_cfg_fps_change(void *data, Evas_Object *obj, void *event_info) {
@@ -802,7 +840,9 @@ struct _Gadit
 E_Slipshelf *local_slipshelf = NULL;
 Eina_List *gadits = NULL;
 Ecore_Timer *_e_cfg_gadgets_change_timer = NULL;
-static int _e_cfg_gadgets_change_timeout(void *data) {
+static Eina_Bool
+_e_cfg_gadgets_change_timeout(__UNUSED__ void *data)
+{
    Eina_List *l2;
 	Gadit *gi;
    int update = 0;
@@ -835,7 +875,8 @@ static int _e_cfg_gadgets_change_timeout(void *data) {
         e_gadcon_populate(local_slipshelf->gadcon_extra);
      }
    e_config_save_queue();
-   _e_cfg_gadgets_change_timer = NULL; return 0;
+   _e_cfg_gadgets_change_timer = NULL;
+   return ECORE_CALLBACK_CANCEL;
 }
 static void
 _e_cfg_gadgets_change(void *data, Evas_Object *obj, void *event_info) {
@@ -965,7 +1006,8 @@ e_cfg_gadgets(E_Container *con, const char *params)
 int external_keyboard = 0;
 Ecore_Timer *_e_cfg_keyboard_change_timer = NULL;
 
-static int _e_cfg_keyboard_change_timeout(void *data)
+static Eina_Bool
+_e_cfg_keyboard_change_timeout(__UNUSED__ void *data)
 {
    Eina_List *l;
 
@@ -993,7 +1035,7 @@ static int _e_cfg_keyboard_change_timeout(void *data)
 	if (kbds)
 	  {
 	     nn = 2;
-	     EINA_LIST_FOREACH(kbds, l, desktop)
+	     EINA_LIST_FREE(kbds, desktop)
 	       {
                   const char *dname;
 		  
@@ -1004,13 +1046,15 @@ static int _e_cfg_keyboard_change_timeout(void *data)
 			 illume_cfg->kbd.run_keyboard = eina_stringshare_add(dname);
 		       break;
 		    }
+		  efreet_desktop_free(desktop);
 		  nn++;
 	       }
 	  }
      }
    e_mod_win_cfg_kbd_update();
    e_config_save_queue();
-   _e_cfg_keyboard_change_timer = NULL; return 0;
+   _e_cfg_keyboard_change_timer = NULL;
+   return ECORE_CALLBACK_CANCEL;
 }
 static void
 _e_cfg_keyboard_change(void *data, Evas_Object *obj, void *event_info) {
@@ -1058,7 +1102,7 @@ _e_cfg_keyboard_ui(E_Config_Dialog *cfd, Evas *e, E_Config_Dialog_Data *cfdata)
 	if (kbds)
 	  {
 	     nn = 2;
-	     EINA_LIST_FOREACH(kbds, l, desktop)
+	     EINA_LIST_FREE(kbds, desktop)
 	       {
                   const char *dname;
 		  
@@ -1071,6 +1115,7 @@ _e_cfg_keyboard_ui(E_Config_Dialog *cfd, Evas *e, E_Config_Dialog_Data *cfdata)
 			    break;
 			 }
 		    }
+		  efreet_desktop_free(desktop);
 		  nn++;
 	       }
 	  }
@@ -1090,14 +1135,12 @@ _e_cfg_keyboard_ui(E_Config_Dialog *cfd, Evas *e, E_Config_Dialog_Data *cfdata)
 	int nn = 2;
 	
 	kbds = efreet_util_desktop_category_list("Keyboard");
-	EINA_LIST_FOREACH(kbds, l, desktop)
+	EINA_LIST_FREE(kbds, desktop)
 	       {
-                  const char *dname;
-		  
-		  dname = ecore_file_file_get(desktop->orig_path);
 		  o = e_widget_radio_add(e, desktop->name, nn, rg);
 		  e_widget_framelist_object_append(frame, o);
 		  evas_object_smart_callback_add(o, "changed", _e_cfg_keyboard_change, NULL);
+		  efreet_desktop_free(desktop);
 		  nn++;
 	       }
 	  }
@@ -1184,7 +1227,7 @@ _e_cfg_win_complete(Evas_Object *ol)
    win = evas_object_data_get(ol, "win");
    d = win->data;
 
-   e_widget_min_size_get(ol, &mw, &mh);
+   e_widget_size_min_get(ol, &mw, &mh);
    evas_object_resize(ol, mw, mh);
 
    sf = e_widget_scrollframe_simple_add(evas_object_evas_get(ol), ol);
@@ -1374,6 +1417,40 @@ _dbcb_launcher_single_click_set(E_DBus_Object *obj, DBusMessage *msg)
    return reply;
 }
 
+// illume_cfg->launcher.single_click_delay
+static DBusMessage *
+_dbcb_launcher_single_click_delay_get(E_DBus_Object *obj, DBusMessage *msg)
+{
+   DBusMessage *reply;
+   DBusMessageIter iter;
+
+   reply = dbus_message_new_method_return(msg);
+   dbus_message_iter_init_append(reply, &iter);
+   dbus_message_iter_append_basic(&iter, DBUS_TYPE_INT32, &(illume_cfg->launcher.single_click_delay));
+   return reply;
+}
+
+static DBusMessage *
+_dbcb_launcher_single_click_delay_set(E_DBus_Object *obj, DBusMessage *msg)
+{
+   DBusMessageIter iter;
+   DBusMessage *reply;
+   int val;
+   
+   dbus_message_iter_init(msg, &iter);
+   dbus_message_iter_get_basic(&iter, &(val));
+   if (val >= 0)
+     {
+	illume_cfg->launcher.single_click_delay = val;
+	_e_cfg_launcher_change(NULL, NULL, NULL);
+	return dbus_message_new_method_return(msg);
+     }
+   reply = dbus_message_new_error(msg,
+				  "org.enlightenment.DBus.InvalidArgument",
+				  "Parameter must be greater than 0");
+   return reply;
+}
+
 // e_config->screensaver_timeout 0(off)-3600
 static DBusMessage *
 _dbcb_screensaver_timeout_get(E_DBus_Object *obj, DBusMessage *msg)
@@ -1920,7 +1997,7 @@ _dbcb_keyboard_get(E_DBus_Object *obj, DBusMessage *msg)
    else if ((illume_cfg->kbd.use_internal) && (!illume_cfg->kbd.run_keyboard))
      s = "internal";
    else if (illume_cfg->kbd.run_keyboard)
-     s = illume_cfg->kbd.run_keyboard;
+     s = (char *)illume_cfg->kbd.run_keyboard;
    dbus_message_iter_append_basic(&iter, DBUS_TYPE_STRING, &(s));
    return reply;
 }
@@ -1967,6 +2044,8 @@ static const DB_Method methods[] =
    {"LauncherIconSizeSet", "i", "", _dbcb_launcher_icon_size_set},
    {"LauncherSingleClickGet", "", "i", _dbcb_launcher_single_click_get},
    {"LauncherSingleClickSet", "i", "", _dbcb_launcher_single_click_set},
+   {"LauncherSingleClickDelayGet", "", "i", _dbcb_launcher_single_click_delay_get},
+   {"LauncherSingleClickDelaySet", "i", "", _dbcb_launcher_single_click_delay_set},
    {"ScreensaverTimeoutGet", "", "i", _dbcb_screensaver_timeout_get},
    {"ScreensaverTimeoutSet", "i", "", _dbcb_screensaver_timeout_set},
    {"AutosuspendTimeoutGet", "", "i", _dbcb_autosuspend_timeout_get},
