@@ -79,6 +79,7 @@ struct _Instance
       Ecore_Event_Handler *show;
       Ecore_Event_Handler *reparent;
       Ecore_Event_Handler *sel_clear;
+      Ecore_Event_Handler *configure;
    } handler;
    struct
    {
@@ -86,8 +87,8 @@ struct _Instance
    } timer;
    struct
    {
-      Ecore_Idler *size_apply;
-   } idler;
+      Ecore_Job *size_apply;
+   } job;
    Eina_List *icons;
    E_Menu *menu;
 };
@@ -159,8 +160,6 @@ _systray_menu_new(Instance *inst, Evas_Event_Mouse_Down *ev)
    e_gadcon_canvas_zone_geometry_get(inst->gcc->gadcon, &x, &y, NULL, NULL);
    e_menu_activate_mouse(mn, zone, x + ev->output.x, y + ev->output.y,
 			 1, 1, E_MENU_POP_DIRECTION_AUTO, ev->timestamp);
-   evas_event_feed_mouse_up(inst->evas, ev->button,
-			    EVAS_BUTTON_NONE, ev->timestamp, NULL);
 }
 
 static void
@@ -187,6 +186,11 @@ _systray_size_apply_do(Instance *inst)
    if (w < 1) w = 1;
    if (h < 1) h = 1;
 
+   if (eina_list_count(inst->icons) == 0)
+      ecore_x_window_hide(inst->win.base);
+   else
+      ecore_x_window_show(inst->win.base);
+
    e_gadcon_client_aspect_set(inst->gcc, w, h);
    e_gadcon_client_min_size_set(inst->gcc, w, h);
 
@@ -194,20 +198,19 @@ _systray_size_apply_do(Instance *inst)
    ecore_x_window_move_resize(inst->win.base, x, y, w, h);
 }
 
-static int
+static void
 _systray_size_apply_delayed(void *data)
 {
    Instance *inst = data;
    _systray_size_apply_do(inst);
-   inst->idler.size_apply = NULL;
-   return 0;
+   inst->job.size_apply = NULL;
 }
 
 static void
 _systray_size_apply(Instance *inst)
 {
-   if (inst->idler.size_apply) return;
-   inst->idler.size_apply = ecore_idler_add(_systray_size_apply_delayed, inst);
+   if (inst->job.size_apply) return;
+   inst->job.size_apply = ecore_job_add(_systray_size_apply_delayed, inst);
 }
 
 static void
@@ -357,7 +360,7 @@ _systray_icon_add(Instance *inst, const Ecore_X_Window win)
    ecore_x_window_save_set_add(win);
    ecore_x_window_shape_events_select(win, 1);
 
-   ecore_x_window_geometry_get(win, NULL, NULL, &w, &h);
+   //ecore_x_window_geometry_get(win, NULL, NULL, &w, &h);
 
    evas_object_event_callback_add
      (o, EVAS_CALLBACK_MOVE, _systray_icon_cb_move, icon);
@@ -479,7 +482,8 @@ _systray_base_create(Instance *inst)
    evas_object_geometry_get(o, &x, &y, &w, &h);
    if (w < 1) w = 1;
    if (h < 1) h = 1;
-   inst->win.base = ecore_x_window_new(inst->win.parent, x, y, w, h);
+   inst->win.base = ecore_x_window_new(0, 0, 0, w, h);
+   ecore_x_window_reparent(inst->win.base, inst->win.parent, x, y); 
    ecore_x_window_background_color_set(inst->win.base, r, g, b);
    ecore_x_window_show(inst->win.base);
    return 1;
@@ -540,7 +544,7 @@ _systray_activate(Instance *inst)
    return 1;
 }
 
-static int
+static Eina_Bool
 _systray_activate_retry(void *data)
 {
    Instance *inst = data;
@@ -555,10 +559,10 @@ _systray_activate_retry(void *data)
 	     RETRY_TIMEOUT);
 
    if (!ret)
-     return 1;
+     return ECORE_CALLBACK_RENEW;
 
    inst->timer.retry = NULL;
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
 static void
@@ -569,7 +573,7 @@ _systray_retry(Instance *inst)
      (RETRY_TIMEOUT, _systray_activate_retry, inst);
 }
 
-static int
+static Eina_Bool
 _systray_activate_retry_first(void *data)
 {
    Instance *inst = data;
@@ -581,7 +585,7 @@ _systray_activate_retry_first(void *data)
      {
 	fputs("SYSTRAY: activate success!\n", stderr);
 	inst->timer.retry = NULL;
-	return 0;
+	return ECORE_CALLBACK_CANCEL;
      }
 
    edje_object_signal_emit(inst->ui.gadget, _sig_disable, _sig_source);
@@ -591,7 +595,7 @@ _systray_activate_retry_first(void *data)
 
    inst->timer.retry = NULL;
    _systray_retry(inst);
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
 static void
@@ -691,7 +695,7 @@ _systray_handle_xembed(Instance *inst __UNUSED__, Ecore_X_Event_Client_Message *
      }
 }
 
-static int
+static Eina_Bool
 _systray_cb_client_message(void *data, int type __UNUSED__, void *event)
 {
    Ecore_X_Event_Client_Message *ev = event;
@@ -704,10 +708,10 @@ _systray_cb_client_message(void *data, int type __UNUSED__, void *event)
    else if (ev->message_type == _atom_xembed)
      _systray_handle_xembed(inst, ev);
 
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
+static Eina_Bool
 _systray_cb_window_destroy(void *data, int type __UNUSED__, void *event)
 {
    Ecore_X_Event_Window_Destroy *ev = event;
@@ -722,10 +726,10 @@ _systray_cb_window_destroy(void *data, int type __UNUSED__, void *event)
 	  break;
        }
 
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
+static Eina_Bool
 _systray_cb_window_show(void *data, int type __UNUSED__, void *event)
 {
    Ecore_X_Event_Window_Show *ev = event;
@@ -740,10 +744,28 @@ _systray_cb_window_show(void *data, int type __UNUSED__, void *event)
 	  break;
        }
 
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
+static Eina_Bool
+_systray_cb_window_configure(void *data, int type __UNUSED__, void *event)
+{
+   Ecore_X_Event_Window_Configure *ev = event;
+   Instance *inst = data;
+   Icon *icon;
+   const Eina_List *l;
+
+   EINA_LIST_FOREACH(inst->icons, l, icon)
+     if (icon->win == ev->win)
+       {
+	  _systray_icon_geometry_apply(icon);
+	  break;
+       }
+
+   return ECORE_CALLBACK_PASS_ON;
+}
+
+static Eina_Bool
 _systray_cb_reparent_notify(void *data, int type __UNUSED__, void *event)
 {
    Ecore_X_Event_Window_Reparent *ev = event;
@@ -758,10 +780,10 @@ _systray_cb_reparent_notify(void *data, int type __UNUSED__, void *event)
 	  break;
        }
 
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-static int
+static Eina_Bool
 _systray_cb_selection_clear(void *data, int type __UNUSED__, void *event)
 {
    Ecore_X_Event_Selection_Clear *ev = event;
@@ -781,7 +803,7 @@ _systray_cb_selection_clear(void *data, int type __UNUSED__, void *event)
 	inst->win.base = None;
 	_systray_retry(inst);
      }
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
 static void
@@ -792,7 +814,7 @@ _systray_theme(Evas_Object *o, const char *shelf_style, const char *gc_style)
    char buf[128], *p;
    size_t len, avail;
 
-   len = ecore_strlcpy(buf, _group_gadget, sizeof(buf));
+   len = eina_strlcpy(buf, _group_gadget, sizeof(buf));
    if (len >= sizeof(buf))
      goto fallback;
    p = buf + len;
@@ -811,7 +833,7 @@ _systray_theme(Evas_Object *o, const char *shelf_style, const char *gc_style)
    if (shelf_style)
      {
 	size_t r;
-	r = ecore_strlcpy(p, shelf_style, avail);
+	r = eina_strlcpy(p, shelf_style, avail);
 	if (r < avail && e_theme_edje_object_set(o, base_theme, buf))
 	  return;
      }
@@ -819,7 +841,7 @@ _systray_theme(Evas_Object *o, const char *shelf_style, const char *gc_style)
    if (gc_style)
      {
 	size_t r;
-	r = ecore_strlcpy(p, gc_style, avail);
+	r = eina_strlcpy(p, gc_style, avail);
 	if (r < avail && e_theme_edje_object_set(o, base_theme, buf))
 	  return;
      }
@@ -838,7 +860,7 @@ _systray_theme(Evas_Object *o, const char *shelf_style, const char *gc_style)
    if (shelf_style)
      {
 	size_t r;
-	r = ecore_strlcpy(p, shelf_style, avail);
+	r = eina_strlcpy(p, shelf_style, avail);
 	if (r < avail && edje_object_file_set(o, path, buf))
 	  return;
      }
@@ -846,7 +868,7 @@ _systray_theme(Evas_Object *o, const char *shelf_style, const char *gc_style)
    if (gc_style)
      {
 	size_t r;
-	r = ecore_strlcpy(p, gc_style, avail);
+	r = eina_strlcpy(p, gc_style, avail);
 	if (r < avail && edje_object_file_set(o, path, buf))
 	  return;
      }
@@ -932,6 +954,8 @@ _gc_init(E_Gadcon *gc, const char *name, const char *id, const char *style)
      (ECORE_X_EVENT_WINDOW_REPARENT, _systray_cb_reparent_notify, inst);
    inst->handler.sel_clear = ecore_event_handler_add
      (ECORE_X_EVENT_SELECTION_CLEAR, _systray_cb_selection_clear, inst);
+   inst->handler.configure = ecore_event_handler_add
+     (ECORE_X_EVENT_WINDOW_CONFIGURE, _systray_cb_window_configure, inst);
 
    instance = inst;
    return inst->gcc;
@@ -967,10 +991,12 @@ _gc_shutdown(E_Gadcon_Client *gcc)
      ecore_event_handler_del(inst->handler.reparent);
    if (inst->handler.sel_clear)
      ecore_event_handler_del(inst->handler.sel_clear);
+   if (inst->handler.configure)
+     ecore_event_handler_del(inst->handler.configure);
    if (inst->timer.retry)
      ecore_timer_del(inst->timer.retry);
-   if (inst->idler.size_apply)
-     ecore_idler_del(inst->idler.size_apply);
+   if (inst->job.size_apply)
+     ecore_job_del(inst->job.size_apply);
 
    if (instance == inst)
      instance = NULL;
@@ -1092,7 +1118,10 @@ _gc_id_new(E_Gadcon_Client_Class *client_class __UNUSED__)
 static const E_Gadcon_Client_Class _gc_class =
 {
   GADCON_CLIENT_CLASS_VERSION, _name,
-  {_gc_init, _gc_shutdown, _gc_orient, _gc_label, _gc_icon, _gc_id_new, NULL},
+  {
+     _gc_init, _gc_shutdown, _gc_orient, _gc_label, _gc_icon, _gc_id_new, NULL,
+     e_gadcon_site_is_not_toolbar
+  },
   E_GADCON_CLIENT_STYLE_INSET
 };
 
