@@ -27,7 +27,7 @@ wizard_page_show(E_Wizard_Page *pg)
    of = e_widget_framelist_add(pg->evas, _("Select applications"), 0);
 	
    ob = e_widget_ilist_add(pg->evas, 32 * e_scale, 32 * e_scale, NULL);
-   e_widget_min_size_set(ob, 140 * e_scale, 140 * e_scale);
+   e_widget_size_min_set(ob, 140 * e_scale, 140 * e_scale);
 	
    e_widget_ilist_freeze(ob);
    e_widget_ilist_go(ob);
